# Copyright 2016 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader

MINIMAL_SHADER = "#version 140\nvoid main(){}"
# This one is valid GLSL but not valid HLSL.
GLSL_VERTEX_SHADER = "#version 140\nvoid main(){ gl_Position = vec4(1.0);}"
# This one is valid HLSL but not valid GLSL.
HLSL_VERTEX_SHADER = "float4 EntryPoint() : SV_POSITION { return float4(1.0); }"
HLSL_VERTEX_SHADER_WITH_MAIN = "float4 main() : SV_POSITION { return float4(1.0); }"
HLSL_VERTEX_SHADER_WITH_FOOBAR = "float4 Foobar() : SV_POSITION { return float4(1.0); }"

# Expected assembly code within certain shaders.
ASSEMBLY_ENTRY_POINT = "OpEntryPoint Vertex %EntryPoint \"EntryPoint\""
ASSEMBLY_MAIN = "OpEntryPoint Vertex %main \"main\""
ASSEMBLY_FOOBAR = "OpEntryPoint Vertex %Foobar \"Foobar\""


@inside_glslc_testsuite('OptionFEntryPoint')
class TestEntryPointDefaultsToMainForGlsl(expect.ValidAssemblyFileWithSubstr):
    """Tests that entry point name defaults to "main" in a GLSL shader."""

    shader = FileShader(GLSL_VERTEX_SHADER, '.vert')
    glslc_args = ['-S', shader]
    expected_assembly_substr = ASSEMBLY_MAIN


@inside_glslc_testsuite('OptionFEntryPoint')
class TestEntryPointDefaultsToMainForHlsl(expect.ValidAssemblyFileWithSubstr):
    """Tests that entry point name defaults to "main" in an HLSL shader."""

    shader = FileShader(HLSL_VERTEX_SHADER_WITH_MAIN, '.vert')
    glslc_args = ['-x', 'hlsl', '-S', shader]
    expected_assembly_substr = ASSEMBLY_MAIN


@inside_glslc_testsuite('OptionFEntryPoint')
class TestFEntryPointMainOnGlslShader(expect.ValidAssemblyFileWithSubstr):
    """Tests -fentry-point=main with a GLSL shader."""

    shader = FileShader(GLSL_VERTEX_SHADER, '.vert')
    glslc_args = ['-fentry-point=main', '-S', shader]
    expected_assembly_substr = ASSEMBLY_MAIN


@inside_glslc_testsuite('OptionFEntryPoint')
class TestFEntryPointMainOnHlslShaderNotMatchingSource(expect.ValidObjectFileWithWarning):
    """Tests -x hlsl on an HLSL shader with -fentry-point=main
    not matching the source."""

    shader = FileShader(HLSL_VERTEX_SHADER, '.vert')
    glslc_args = ['-x', 'hlsl', '-fentry-point=main', '-c', shader]
    expected_warning = [shader,
                        ': warning: Linking vertex stage: Entry point not found\n'
                        '1 warning generated.\n']


@inside_glslc_testsuite('OptionFEntryPoint')
class TestFEntryPointSpecifiedOnHlslShaderInDisassembly(expect.ValidObjectFileWithAssemblySubstr):
    """Tests -x hlsl on an HLSL shader with -fentry-point=EntryPoint
    matching source."""

    shader = FileShader(HLSL_VERTEX_SHADER, '.vert', assembly_substr=ASSEMBLY_ENTRY_POINT)
    glslc_args = ['-x', 'hlsl', '-fentry-point=EntryPoint', '-c', shader]


@inside_glslc_testsuite('OptionFEntryPoint')
class TestFEntryPointAffectsSubsequentShaderFiles(expect.ValidObjectFileWithAssemblySubstr):
    """Tests -x hlsl affects several subsequent shader source files."""

    shader1 = FileShader(HLSL_VERTEX_SHADER, '.vert', assembly_substr=ASSEMBLY_ENTRY_POINT)
    shader2 = FileShader(HLSL_VERTEX_SHADER, '.vert', assembly_substr=ASSEMBLY_ENTRY_POINT)
    glslc_args = ['-x', 'hlsl', '-fentry-point=EntryPoint', '-c', shader1, shader2]


@inside_glslc_testsuite('OptionFEntryPoint')
class TestFEntryPointOverridesItself(expect.ValidObjectFileWithAssemblySubstr):
    """Tests that a later -fentry-point option overrides an earlier use."""

    shader = FileShader(HLSL_VERTEX_SHADER, '.vert', assembly_substr=ASSEMBLY_ENTRY_POINT)
    glslc_args = ['-x', 'hlsl', '-fentry-point=foobar', '-fentry-point=EntryPoint',
                  '-c', shader]


@inside_glslc_testsuite('OptionFEntryPoint')
class TestFEntryPointDefaultAndTwoOthers(expect.ValidObjectFileWithAssemblySubstr):
    """Tests three shaders with different entry point names. The first uses "main"
    with default entry point processing, and the remaining shaders get their
    own -fentry-point argument."""

    shaderMain = FileShader(HLSL_VERTEX_SHADER_WITH_MAIN, '.vert',
                            assembly_substr=ASSEMBLY_MAIN)
    shaderEntryPoint = FileShader(HLSL_VERTEX_SHADER, '.vert',
                                  assembly_substr=ASSEMBLY_ENTRY_POINT)
    shaderFoobar = FileShader(HLSL_VERTEX_SHADER_WITH_FOOBAR, '.vert',
                              assembly_substr=ASSEMBLY_FOOBAR)
    glslc_args = ['-x', 'hlsl', '-c', shaderMain,
                  '-fentry-point=EntryPoint', shaderEntryPoint,
                  '-fentry-point=Foobar', shaderFoobar]



# Copyright 2016 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
import re
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader

MINIMAL_SHADER = '#version 140\nvoid main() {}'
# Regular expression patterns for the minimal shader. The magic number should
# match exactly, and there should not be a trailing comma at the end of the
# list. When -mfmt=c is specified, curly brackets should be presented.
MINIMAL_SHADER_NUM_FORMAT_PATTERN = "^0x07230203.*[0-9a-f]$"
MINIMAL_SHADER_C_FORMAT_PATTERN = "^\\{0x07230203.*[0-9a-f]\\}"
ERROR_SHADER = '#version 140\n#error\nvoid main() {}'


@inside_glslc_testsuite('OptionMfmt')
class TestFmtCWorksWithDashC(expect.ValidFileContents):
    """Tests that -mfmt=c works with -c for single input file. SPIR-V binary
    code output should be emitted as a C-style initializer list in the output
    file.
    """
    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = [shader, '-c', '-mfmt=c', '-o', 'output_file']
    target_filename = 'output_file'
    expected_file_contents = re.compile(MINIMAL_SHADER_C_FORMAT_PATTERN, re.S)


@inside_glslc_testsuite('OptionMfmt')
class TestFmtNumWorksWithDashC(expect.ValidFileContents):
    """Tests that -mfmt=num works with -c for single input file. SPIR-V binary
    code output should be emitted as a list of hex numbers in the output file.
    """
    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = [shader, '-c', '-mfmt=num', '-o', 'output_file']
    target_filename = 'output_file'
    expected_file_contents = re.compile(MINIMAL_SHADER_NUM_FORMAT_PATTERN, re.S)


@inside_glslc_testsuite('OptionMfmt')
class TestFmtBinWorksWithDashC(expect.ValidObjectFile):
    """Tests that -mfmt=bin works with -c for single input file. This test
    should simply have the SPIR-V binary generated.
    """
    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = [shader, '-c', '-mfmt=bin']


@inside_glslc_testsuite('OptionMfmt')
class TestFmtCWithLinking(expect.ValidFileContents):
    """Tests that -mfmt=c works when linkding is enabled (no -c specified).
    SPIR-V binary code should be emitted as a C-style initializer list in the
    output file.
    """
    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = [shader, '-mfmt=c']
    target_filename = 'a.spv'
    expected_file_contents = re.compile(MINIMAL_SHADER_C_FORMAT_PATTERN, re.S)


@inside_glslc_testsuite('OptionMfmt')
class TestFmtNumWithLinking(expect.ValidFileContents):
    """Tests that -mfmt=num works when linkding is enabled (no -c specified).
    SPIR-V binary code should be emitted as a C-style initializer list in the
    output file.
    """
    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = [shader, '-mfmt=num']
    target_filename = 'a.spv'
    expected_file_contents = re.compile(MINIMAL_SHADER_NUM_FORMAT_PATTERN, re.S)


@inside_glslc_testsuite('OptionMfmt')
class TestFmtCErrorWhenOutputDisasembly(expect.ErrorMessage):
    """Tests that specifying '-mfmt=c' when the compiler is set to
    disassembly mode should trigger an error.
    """
    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = [shader, '-mfmt=c', '-S', '-o', 'output_file']
    expected_error = ("glslc: error: cannot emit output as a C-style "
                      "initializer list when only preprocessing the source\n")


@inside_glslc_testsuite('OptionMfmt')
class TestFmtNumErrorWhenOutputDisasembly(expect.ErrorMessage):
    """Tests that specifying '-mfmt=num' when the compiler is set to
    disassembly mode should trigger an error.
    """
    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = [shader, '-mfmt=num', '-S', '-o', 'output_file']
    expected_error = (
        "glslc: error: cannot emit output as a list of hex numbers "
        "when only preprocessing the source\n")


@inside_glslc_testsuite('OptionMfmt')
class TestFmtBinErrorWhenOutputDisasembly(expect.ErrorMessage):
    """Tests that specifying '-mfmt=bin' when the compiler is set to
    disassembly mode should trigger an error.
    """
    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = [shader, '-mfmt=bin', '-S', '-o', 'output_file']
    expected_error = ("glslc: error: cannot emit output as a binary "
                      "when only preprocessing the source\n")


@inside_glslc_testsuite('OptionMfmt')
class TestFmtNumErrorWhenOutputPreprocess(expect.ErrorMessage):
    """Tests that specifying '-mfmt=num' when the compiler is set to
    preprocessing only mode should trigger an error.
    """
    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = [shader, '-mfmt=num', '-E', '-o', 'output_file']
    expected_error = (
        "glslc: error: cannot emit output as a list of hex numbers "
        "when only preprocessing the source\n")


@inside_glslc_testsuite('OptionMfmt')
class TestFmtCErrorWithDashCapM(expect.ErrorMessage):
    """Tests that specifying '-mfmt=c' should trigger an error when the
    compiler is set to dump dependency info as the output (-M or -MM is
    specified).
    """
    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = [shader, '-mfmt=c', '-M', '-o', 'output_file']
    expected_error = ("glslc: error: cannot emit output as a C-style "
                      "initializer list when only preprocessing the source\n")


@inside_glslc_testsuite('OptionMfmt')
class TestFmtCWorksWithDashCapMD(expect.ValidFileContents):
    """Tests that -mfmt=c works with '-c -MD'. SPIR-V binary code
    should be emitted as a C-style initializer list in the output file.
    """
    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = [shader, '-mfmt=c', '-c', '-MD', '-o', 'output_file']
    target_filename = 'output_file'
    expected_file_contents = re.compile(MINIMAL_SHADER_C_FORMAT_PATTERN, re.S)


@inside_glslc_testsuite('OptionMfmt')
class TestFmtNumWorksWithDashCapMD(expect.ValidFileContents):
    """Tests that -mfmt=num works with '-c -MD'. SPIR-V binary code
    should be emitted as a C-style initializer list in the output file.
    """
    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = [shader, '-mfmt=num', '-c', '-MD', '-o', 'output_file']
    target_filename = 'output_file'
    expected_file_contents = re.compile(MINIMAL_SHADER_NUM_FORMAT_PATTERN, re.S)


@inside_glslc_testsuite('OptionMfmt')
class TestFmtCExitsElegantlyWithErrorInShader(expect.ErrorMessage):
    """Tests that the compiler fails elegantly with -mfmt=c when there are
    errors in the input shader.
    """
    shader = FileShader(ERROR_SHADER, '.vert')
    glslc_args = [shader, '-mfmt=c']
    expected_error = [shader, ':3: error: \'#error\' :\n',
                      '1 error generated.\n']


@inside_glslc_testsuite('OptionMfmt')
class TestFmtNumExitsElegantlyWithErrorInShader(expect.ErrorMessage):
    """Tests that the compiler fails elegantly with -mfmt=num when there are
    errors in the input shader.
    """
    shader = FileShader(ERROR_SHADER, '.vert')
    glslc_args = [shader, '-mfmt=num']
    expected_error = [shader, ':3: error: \'#error\' :\n',
                      '1 error generated.\n']


@inside_glslc_testsuite('OptionMfmt')
class TestFmtBinExitsElegantlyWithErrorInShader(expect.ErrorMessage):
    """Tests that the compiler fails elegantly with -mfmt=binary when there are
    errors in the input shader.
    """
    shader = FileShader(ERROR_SHADER, '.vert')
    glslc_args = [shader, '-mfmt=bin']
    expected_error = [shader, ':3: error: \'#error\' :\n',
                      '1 error generated.\n']



# Copyright 2025 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader

COMPUTE_SHADER = """#version 450
#extension GL_EXT_control_flow_attributes : require

layout(local_size_x=1) in;

layout(binding=0) buffer Out { uint x; } g_out;

void main() {
    uint x = 0;
    [[unroll]]
    for (int i = 0; i < 100; ++i) {
        x = x + uint(i);
    }
    g_out.x = x;
}
"""

@inside_glslc_testsuite('OptionFMaxIdBound')
class TestFMaxIdBoundLow(expect.ErrorMessageSubstr):
    """Tests that compilation fails with a low -fmax-id-bound."""

    shader = FileShader(COMPUTE_SHADER, ".comp")
    glslc_args = ['-c', shader, '-fmax-id-bound=300', '-O']
    expected_error_substr = [" ID overflow. Try running compact-ids"]


@inside_glslc_testsuite('OptionFMaxIdBound')
class TestFMaxIdBoundHigh(expect.ValidObjectFile):
    """Tests that compilation passes with a high -fmax-id-bound."""

    shader = FileShader(COMPUTE_SHADER, ".comp")
    glslc_args = ['-c', shader, '-fmax-id-bound=200000', '-O']



# Copyright 2017 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader


# A GLSL shader with uniforms without explicit bindings.
GLSL_SHADER = """#version 450
  layout(set=0, binding=0)
  buffer B { float x; vec3 y; } my_ssbo;
  void main() {
    my_ssbo.x = 1.0;
  }"""


@inside_glslc_testsuite('OptionFHlslOffsets')
class StandardOffsetsByDefault(expect.ValidAssemblyFileWithSubstr):
    """Tests that standard GLSL packign is used by default."""

    shader = FileShader(GLSL_SHADER, '.vert')
    glslc_args = ['-S', shader]
    expected_assembly_substr = "OpMemberDecorate %B 1 Offset 16"


@inside_glslc_testsuite('OptionFHlslOffsets')
class HlslOffsetsWhenRequested(expect.ValidAssemblyFileWithSubstr):
    """Tests that standard GLSL packign is used by default."""

    shader = FileShader(GLSL_SHADER, '.vert')
    glslc_args = ['-S', '-fhlsl-offsets', shader]
    expected_assembly_substr = "OpMemberDecorate %B 1 Offset 4"



# Copyright 2015 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader


def core_vert_shader_without_version():
    # gl_ClipDistance doesn't exist in es profile (at least until 3.10).
    return 'void main() { gl_ClipDistance[0] = 5.; }'


def core_frag_shader_without_version():
    # gl_SampleID appears in core profile from 4.00.
    # gl_sampleID doesn't exsit in es profile (at least until 3.10).
    return 'void main() { int temp = gl_SampleID; }'


def hlsl_compute_shader_with_barriers():
    # Use "main" to avoid the need for -fentry-point
    return 'void main() { AllMemoryBarrierWithGroupSync(); }'


@inside_glslc_testsuite('OptionStd')
class TestStdNoArg(expect.ErrorMessage):
    """Tests -std alone."""

    glslc_args = ['-std']
    expected_error = ["glslc: error: unknown argument: '-std'\n"]


@inside_glslc_testsuite('OptionStd')
class TestStdEqNoArg(expect.ErrorMessage):
    """Tests -std= with no argument."""

    glslc_args = ['-std=']
    expected_error = ["glslc: error: invalid value '' in '-std='\n"]


@inside_glslc_testsuite('OptionStd')
class TestStdEqSpaceArg(expect.ErrorMessage):
    """Tests -std= <version-profile>."""

    shader = FileShader(core_frag_shader_without_version(), '.frag')
    glslc_args = ['-c', '-std=', '450core', shader]
    expected_error = ["glslc: error: invalid value '' in '-std='\n"]


# TODO(dneto): The error message changes with different versions of glslang.
@inside_glslc_testsuite('OptionStd')
class TestMissingVersionAndStd(expect.ErrorMessageSubstr):
    """Tests that missing both #version and -std results in errors."""

    shader = FileShader(core_frag_shader_without_version(), '.frag')
    glslc_args = ['-c', shader]
    expected_error_substr = ['error:']


@inside_glslc_testsuite('OptionStd')
class TestMissingVersionButHavingStd(expect.ValidObjectFile):
    """Tests that correct -std fixes missing #version."""

    shader = FileShader(core_frag_shader_without_version(), '.frag')
    glslc_args = ['-c', '-std=450core', shader]


@inside_glslc_testsuite('OptionStd')
class TestGLSL460(expect.ValidObjectFile):
    """Tests that GLSL version 4.6 is supported."""

    shader = FileShader(core_frag_shader_without_version(), '.frag')
    glslc_args = ['-c', '-std=460', shader]


@inside_glslc_testsuite('OptionStd')
class TestGLSL460Core(expect.ValidObjectFile):
    """Tests that GLSL version 4.6 core profile is supported."""

    shader = FileShader(core_frag_shader_without_version(), '.frag')
    glslc_args = ['-c', '-std=460core', shader]


@inside_glslc_testsuite('OptionStd')
class TestESSL320(expect.ValidObjectFile):
    """Tests that ESSL version 3.2 is supported."""

    shader = FileShader(core_frag_shader_without_version(), '.frag')
    glslc_args = ['-c', '-std=320es', shader]


@inside_glslc_testsuite('OptionStd')
class TestStdIgnoredInHlsl(expect.ValidObjectFile):
    """Tests HLSL compilation ignores -std."""

    # Compute shaders are not available in OpenGL 150
    shader = FileShader(hlsl_compute_shader_with_barriers(), '.comp')
    glslc_args = ['-c', '-x', 'hlsl', '-std=150', shader]


@inside_glslc_testsuite('OptionStd')
class TestMissingVersionAndWrongStd(expect.ErrorMessage):
    """Tests missing #version and wrong -std results in errors."""

    shader = FileShader(core_frag_shader_without_version(), '.frag')
    glslc_args = ['-c', '-std=310es', shader]
    expected_error = [
        shader, ":1: error: 'gl_SampleID' : required extension not requested: "
        'GL_OES_sample_variables\n1 error generated.\n']


@inside_glslc_testsuite('OptionStd')
class TestConflictingVersionAndStd(expect.ValidObjectFileWithWarning):
    """Tests that with both #version and -std, -std takes precedence."""

    # Wrong #version here on purpose.
    shader = FileShader(
        '#version 310 es\n' + core_frag_shader_without_version(), '.frag')
    # -std overwrites the wrong #version.
    glslc_args = ['-c', '-std=450core', shader]

    expected_warning = [
        shader, ': warning: (version, profile) forced to be (450, core), while '
        'in source code it is (310, es)\n1 warning generated.\n']


@inside_glslc_testsuite('OptionStd')
class TestMultipleStd(expect.ValidObjectFile):
    """Tests that for multiple -std, the last one takes effect."""

    shader = FileShader(core_frag_shader_without_version(), '.frag')
    glslc_args = ['-c', '-std=100', '-std=310es', shader, '-std=450core']


@inside_glslc_testsuite('OptionStd')
class TestMultipleFiles(expect.ValidObjectFileWithWarning):
    """Tests that -std covers all files."""

    shader1 = FileShader(core_frag_shader_without_version(), '.frag')
    shader2 = FileShader(core_vert_shader_without_version(), '.vert')
    shader3 = FileShader(
        '#version 310 es\n' + core_frag_shader_without_version(), '.frag')
    glslc_args = ['-c', '-std=450core', shader1, shader2, shader3]

    expected_warning = [
        shader3, ': warning: (version, profile) forced to be (450, '
        'core), while in source code it is (310, es)\n'
        '1 warning generated.\n']


@inside_glslc_testsuite('OptionStd')
class TestUnkownProfile(expect.ErrorMessage):
    """Tests that -std rejects unknown profile."""

    shader = FileShader(core_frag_shader_without_version(), '.frag')
    glslc_args = ['-c', '-std=450google', shader]
    expected_error = [
        "glslc: error: invalid value '450google' in '-std=450google'\n"]


@inside_glslc_testsuite('OptionStd')
class TestUnkownVersion(expect.ErrorMessage):
    """Tests that -std rejects unknown version."""

    shader = FileShader(core_frag_shader_without_version(), '.frag')
    glslc_args = ['-c', '-std=42core', shader]
    expected_error = [
        "glslc: error: invalid value '42core' in '-std=42core'\n"]


@inside_glslc_testsuite('OptionStd')
class TestTotallyWrongStdValue(expect.ErrorMessage):
    """Tests that -std rejects totally wrong -std value."""

    shader = FileShader(core_vert_shader_without_version(), '.vert')
    glslc_args = ['-c', '-std=wrong42', shader]

    expected_error = [
        "glslc: error: invalid value 'wrong42' in '-std=wrong42'\n"]


@inside_glslc_testsuite('OptionStd')
class TestVersionInsideSlashSlashComment(expect.ValidObjectFileWithWarning):
    """Tests that -std substitutes the correct #version string."""

    # The second #version string should be substituted and this shader
    # should compile successfully with -std=450core.
    shader = FileShader(
        '// #version 310 es\n#version 310 es\n' +
        core_vert_shader_without_version(), '.vert')
    glslc_args = ['-c', '-std=450core', shader]

    expected_warning = [
        shader, ': warning: (version, profile) forced to be (450, core), while '
        'in source code it is (310, es)\n1 warning generated.\n']


@inside_glslc_testsuite('OptionStd')
class TestVersionInsideSlashStarComment(expect.ValidObjectFileWithWarning):
    """Tests that -std substitutes the correct #version string."""

    # The second #version string should be substituted and this shader
    # should compile successfully with -std=450core.
    shader = FileShader(
        '/* #version 310 es */\n#version 310 es\n' +
        core_vert_shader_without_version(), '.vert')
    glslc_args = ['-c', '-std=450core', shader]

    expected_warning = [
        shader, ': warning: (version, profile) forced to be (450, core), while '
        'in source code it is (310, es)\n1 warning generated.\n']


@inside_glslc_testsuite('OptionStd')
class TestCommentBeforeVersion(expect.ValidObjectFileWithWarning):
    """Tests that comments before #version (same line) is correctly handled."""

    shader = FileShader(
        '/* some comment */ #version 150\n' +
        core_vert_shader_without_version(), '.vert')
    glslc_args = ['-c', '-std=450', shader]

    expected_warning = [
        shader, ': warning: (version, profile) forced to be (450, none), while '
        'in source code it is (150, none)\n1 warning generated.\n']


@inside_glslc_testsuite('OptionStd')
class TestCommentAfterVersion(expect.ValidObjectFileWithWarning):
    """Tests that multiple-line comments after #version is correctly handled."""

    shader = FileShader(
        '#version 150 compatibility ' +
        '/* start \n second line \n end */\n' +
        core_vert_shader_without_version(), '.vert')
    glslc_args = ['-c', '-std=450core', shader]

    expected_warning = [
        shader, ': warning: (version, profile) forced to be (450, core), while '
        'in source code it is (150, compatibility)\n1 warning generated.\n']


# The following test case is disabled because of a bug in glslang.
# When checking non-newline whitespaces, glslang only recognizes
# ' ' and '\t', leaving '\v' and '\f' unhandled. The following test
# case exposes this problem. It should be turned on once a fix for
# glslang is landed.
#@inside_glslc_testsuite('OptionStd')
class TestSpaceAroundVersion(expect.ValidObjectFileWithWarning):
    """Tests that space around #version is correctly handled."""

    shader = FileShader(
        '\t   \t  # \t \f\f version  \v \t\t  310 \v\v \t  es \n' +
        core_vert_shader_without_version(), '.vert')
    glslc_args = ['-c', '-std=450core', shader]

    expected_warning = [
        shader, ': warning: (version, profile) forced to be (450, core), while '
        'in source code it is (310, es)\n1 warning generated.\n']


@inside_glslc_testsuite('OptionStd')
class TestVersionInsideCrazyComment(expect.ValidObjectFileWithWarning):
    """Tests that -std substitutes the correct #version string."""

    # The fourth #version string should be substituted and this shader
    # should compile successfully with -std=450core.
    shader = FileShader(
        '/* */ /* // /* #version 310 es */\n' +  # /*-style comment
        '// /* */ /* /* // #version 310 es\n' +  # //-style comment
        '///*////*//*/*/ #version 310 es*/\n' +  # //-style comment
        '#version 310 es\n' + core_vert_shader_without_version(), '.vert')
    glslc_args = ['-c', '-std=450core', shader]

    expected_warning = [
        shader, ': warning: (version, profile) forced to be (450, core), while '
        'in source code it is (310, es)\n1 warning generated.\n']


@inside_glslc_testsuite('OptionStd')
class TestVersionMissingProfile(expect.ErrorMessage):
    """Tests that missing required profile in -std results in an error."""

    shader = FileShader('#version 140\nvoid main() {}', '.vert')
    glslc_args = ['-c', '-std=310', shader]

    expected_error = [
        shader, ': error: #version: versions 300, 310, and 320 require ',
        "specifying the 'es' profile\n1 error generated.\n"]


@inside_glslc_testsuite('OptionStd')
class TestVersionRedundantProfile(expect.ErrorMessageSubstr):
    """Tests that adding non-required profile in -std results in an error."""

    shader = FileShader('#version 140\nvoid main() {}', '.vert')
    glslc_args = ['-c', '-std=100core', shader]

    expected_error_substr = [
        shader, ': error: #version: versions before 150 do not allow '
        'a profile token\n']



# Copyright 2019 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader


def vulkan_vertex_shader():
    return """#version 310 es
void main() { int t = gl_VertexIndex; }"""


def vulkan_compute_subgroup_shader():
    """Returns a compute shader that requires Vulkan 1.1 and SPIR-V 1.3"""
    return """#version 450
              #extension GL_KHR_shader_subgroup_basic : enable
              void main() { subgroupBarrier(); }"""


@inside_glslc_testsuite('OptionTargetSpv')
class TestDefaultTargetSpvWithVulkanShader(expect.ValidObjectFile):
    """Tests that compiling a Vulkan-specific shader with a default
    target environment succeeds"""
    shader = FileShader(vulkan_vertex_shader(), '.vert')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('OptionTargetSpv')
class TestDefaultTargetSpvWithShaderRequiringSpv1p3Fails(expect.ErrorMessageSubstr):
    """Tests that compiling a shader requiring SPIR-V 1.3 with default SPIR-V
    target should fail.
    """
    shader = FileShader(vulkan_compute_subgroup_shader(), '.comp')
    glslc_args = ['-c', shader]
    expected_error_substr = ["error: 'subgroup op' : requires SPIR-V 1.3\n"]


@inside_glslc_testsuite('OptionTargetSpv')
class TestTargetSpv1p2WithShaderRequiringSpv1p3Fails(expect.ErrorMessageSubstr):
    """Tests that compiling a shader requiring SPIR-V 1.3 but targeting 1.2
    should fail.
    """
    shader = FileShader(vulkan_compute_subgroup_shader(), '.comp')
    glslc_args = ['--target-spv=spv1.2', '-c', shader]
    expected_error_substr = ["error: 'subgroup op' : requires SPIR-V 1.3\n"]


@inside_glslc_testsuite('OptionTargetSpv')
class TestTargetSpv1p3(expect.ValidObjectFile1_3):
    """Tests that compiling to spv1.3 succeeds and generates SPIR-V 1.3 binary."""
    shader = FileShader(vulkan_compute_subgroup_shader(), '.comp')
    glslc_args = ['--target-spv=spv1.3', '-c', shader]


@inside_glslc_testsuite('OptionTargetSpv')
class TestTargetSpv1p4(expect.ValidObjectFile1_4):
    """Tests that compiling to spv1.4 succeeds and generates SPIR-V 1.4 binary."""
    shader = FileShader(vulkan_vertex_shader(), '.vert')
    glslc_args = ['--target-spv=spv1.4', '-c', shader]


@inside_glslc_testsuite('OptionTargetSpv')
class TestTargetSpv1p5(expect.ValidObjectFile1_5):
    """Tests that compiling to spv1.5 succeeds and generates SPIR-V 1.5 binary."""
    shader = FileShader(vulkan_vertex_shader(), '.vert')
    glslc_args = ['--target-spv=spv1.5', '-c', shader]


@inside_glslc_testsuite('OptionTargetSpv')
class TestTargetSpv1p5(expect.ValidObjectFile1_6):
    """Tests that compiling to spv1.6 succeeds and generates SPIR-V 1.6 binary."""
    shader = FileShader(vulkan_vertex_shader(), '.vert')
    glslc_args = ['--target-spv=spv1.6', '-c', shader]


### Option parsing error cases

@inside_glslc_testsuite('OptionTargetSpv')
class TestTargetSpvNoArg(expect.ErrorMessage):
    """Tests the error message of assigning empty string to --target-spv"""
    shader = FileShader(vulkan_vertex_shader(), '.vert')
    glslc_args = ['--target-spv=', shader]
    expected_error = ["glslc: error: invalid value ",
                      "'' in '--target-spv='\n"]


@inside_glslc_testsuite('OptionTargetSpv')
class TestTargetSpvNoEqNoArg(expect.ErrorMessage):
    """Tests the error message of using --target-spv without equal sign and
    arguments"""
    shader = FileShader(vulkan_vertex_shader(), '.vert')
    glslc_args = ['--target-spv', shader]
    expected_error = ["glslc: error: unsupported option: ",
                      "'--target-spv'\n"]


@inside_glslc_testsuite('OptionTargetSpv')
class TestTargetSpvNoEqWithArg(expect.ErrorMessage):
    """Tests the error message of using --target-spv without equal sign but
    arguments"""
    shader = FileShader(vulkan_vertex_shader(), '.vert')
    glslc_args = ['--target-spv', 'spv1.3', shader]
    expected_error = ["glslc: error: unsupported option: ",
                      "'--target-spv'\n"]


@inside_glslc_testsuite('OptionTargetSpv')
class TestTargetSpvEqWrongArg(expect.ErrorMessage):
    """Tests the error message of using --target-spv with wrong argument"""
    shader = FileShader(vulkan_vertex_shader(), '.vert')
    glslc_args = ['--target-spv=wrong_arg', shader]
    expected_error = ["glslc: error: invalid value ",
                      "'wrong_arg' in '--target-spv=wrong_arg'\n"]



# Copyright 2017 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader

# An HLSL shader with uniforms without explicit bindings.
HLSL_SHADER = """
Buffer<float4> t4 : register(t4);
Buffer<float4> t5 : register(t5);

float4 main() : SV_Target0 {
   return float4(t4.Load(0) + t5.Load(1));
}
"""


NEED_THREE_ARGS_ERR = "error: Option -fresource-set-binding requires at least 3 arguments"

@inside_glslc_testsuite('OptionFRegisterSetBinding')
class FRegisterSetBindingForFragRespected(expect.ValidAssemblyFileWithSubstr):
    """Tests -fresource-set-binding on specific shader two textures"""

    shader = FileShader(HLSL_SHADER, '.frag')
    glslc_args = ['-S', '-x', 'hlsl', shader,
                  '-fresource-set-binding', 'frag',
                  't4', '9', '16',
                  't5', '17', '18']
    expected_assembly_substr = """OpDecorate %t4 Binding 16
               OpDecorate %t4 DescriptorSet 9
               OpDecorate %t5 Binding 18
               OpDecorate %t5 DescriptorSet 17"""


@inside_glslc_testsuite('OptionFRegisterSetBinding')
class FRegisterSetBindingForFragRespectedJustOneTriple(expect.ValidAssemblyFileWithSubstr):
    """Tests -fresource-set-binding on specific shader just one texture specified."""

    shader = FileShader(HLSL_SHADER, '.frag')
    glslc_args = ['-S', '-x', 'hlsl', shader,
                  '-fresource-set-binding', 'frag',
                  't4', '9', '16']
    expected_assembly_substr = """OpDecorate %t4 Binding 16
               OpDecorate %t4 DescriptorSet 9
               OpDecorate %t5 Binding 5
               OpDecorate %t5 DescriptorSet 0"""


@inside_glslc_testsuite('OptionFRegisterSetBinding')
class FRegisterSetBindingForWrongStageIgnored(expect.ValidAssemblyFileWithSubstr):
    """Tests -fresource-set-binding on wrong shader ignored"""

    shader = FileShader(HLSL_SHADER, '.frag')
    glslc_args = ['-S', '-x', 'hlsl', shader,
                  '-fresource-set-binding', 'vert',
                  't4', '9', '16',
                  't5', '17', '18']
    expected_assembly_substr = """OpDecorate %t4 Binding 4
               OpDecorate %t4 DescriptorSet 0
               OpDecorate %t5 Binding 5
               OpDecorate %t5 DescriptorSet 0"""


@inside_glslc_testsuite('OptionFRegisterSetBinding')
class FRegisterSetBindingForAllRespected(expect.ValidAssemblyFileWithSubstr):
    """Tests -fresource-set-binding on all stages respected"""

    shader = FileShader(HLSL_SHADER, '.frag')
    glslc_args = ['-S', '-x', 'hlsl', shader,
                  '-fresource-set-binding',
                  't4', '9', '16',
                  't5', '17', '18']
    expected_assembly_substr = """OpDecorate %t4 Binding 16
               OpDecorate %t4 DescriptorSet 9
               OpDecorate %t5 Binding 18
               OpDecorate %t5 DescriptorSet 17"""


@inside_glslc_testsuite('OptionFRegisterSetBinding')
class FRegisterSetBindingTooFewArgs(expect.ErrorMessageSubstr):
    """Tests -fresource-set-binding with too few arguments"""

    shader = FileShader(HLSL_SHADER, '.frag')
    glslc_args = ['-S', '-x', 'hlsl', shader,
                  '-fresource-set-binding', 'frag',
                  't4', '9']
    expected_error_substr = NEED_THREE_ARGS_ERR


@inside_glslc_testsuite('OptionFRegisterSetBinding')
class FRegisterSetBindingInvalidSetNumber(expect.ErrorMessageSubstr):
    """Tests -fresource-set-binding with inavlid set number"""

    shader = FileShader(HLSL_SHADER, '.frag')
    glslc_args = ['-S', '-x', 'hlsl', shader,
                  '-fresource-set-binding', 'frag',
                  't4', '-9', '16']
    expected_error_substr = NEED_THREE_ARGS_ERR


@inside_glslc_testsuite('OptionFRegisterSetBinding')
class FRegisterSetBindingInvalidBindingNumber(expect.ErrorMessageSubstr):
    """Tests -fresource-set-binding with inavlid binding number"""

    shader = FileShader(HLSL_SHADER, '.frag')
    glslc_args = ['-S', '-x', 'hlsl', shader,
                  '-fresource-set-binding', 'frag',
                  't4', '9', '-16']
    expected_error_substr = NEED_THREE_ARGS_ERR



# Copyright 2015 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDNoArg(expect.ErrorMessage):
    """Tests -D without macroname."""

    glslc_args = ['-D']
    expected_error = [
        "glslc: error: argument to '-D' is missing\n",
        'glslc: error: no input files\n']


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDXeqY(expect.ValidObjectFile):
    """Tests -DX=Y."""

    shader = FileShader('#version 150\nvoid main(){X=vec4(1.);}', '.vert')
    glslc_args = ['-c', '-DX=gl_Position', shader]


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDXeq(expect.ValidObjectFile):
    """Tests -DX=."""

    shader = FileShader('#version 150\nvoid main(){X}', '.vert')
    glslc_args = ['-c', '-DX=', shader]


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDX(expect.ValidObjectFile):
    """Tests -DX."""

    shader = FileShader('#version 150\nvoid main(){X}', '.vert')
    glslc_args = ['-c', '-DX', shader]


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDeq(expect.ErrorMessage):
    """Tests -D=.

    This is actually allowed by clang, though the resulting #define
    causes a preprocessing error.
    """

    shader = FileShader('#version 150\nvoid main(){}', '.vert')
    glslc_args = ['-c', '-D=', shader]
    # TODO(antiagainst): figure out what should we report as the line number
    # for errors in predefined macros and fix here.
    expected_error = [
        "<command line>:2: error: '#define' : must be followed by macro name\n",
        '1 error generated.\n']


@inside_glslc_testsuite('OptionCapD')
class TestMultipleDashCapD(expect.ValidObjectFile):
    """Tests multiple -D occurrences."""

    shader = FileShader('#version 150\nvoid main(){X Y a=Z;}', '.vert')
    glslc_args = ['-c', '-DX', '-DY=int', '-DZ=(1+2)', shader]


@inside_glslc_testsuite('OptionCapD')
class TestMultipleDashCapDOfSameName(expect.ValidObjectFile):
    """Tests multiple -D occurrences with same macro name."""

    shader = FileShader('#version 150\nvoid main(){X Y a=Z;}', '.vert')
    glslc_args = ['-c', '-DX=main', '-DY=int', '-DZ=(1+2)', '-DX', shader]


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDGL_(expect.ErrorMessage):
    """Tests that we cannot -D macros beginning with GL_."""

    shader = FileShader('#version 150\nvoid main(){}', '.vert')
    glslc_args = ['-DGL_ES=1', shader]
    expected_error = [
        "glslc: error: names beginning with 'GL_' cannot be "
        'defined: -DGL_ES=1\n']


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDReservedMacro(expect.WarningMessage):
    """Tests that we cannot -D GLSL's predefined macros."""

    shader = FileShader('#version 150\nvoid main(){}', '.vert')
    # Consecutive underscores are banned anywhere in the name.
    glslc_args = [
        '-D__LINE__=1', '-Dmid__dle', '-Dend__', '-D_single_is_valid_', shader]

    w = 'glslc: warning: names containing consecutive underscores are reserved: '
    expected_warning = [w, '-D__LINE__=1\n', w, '-Dmid__dle\n', w, '-Dend__\n']


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithVersion(expect.ErrorMessage):
    """Tests -D works well when #version is present."""

    shader = FileShader(
        """#version 310 es
        void main(){X}
        void foo(){Y}""", '.vert')
    glslc_args = ['-DX=', '-DY=return 3;', shader]

    expected_error = [
        shader, ":3: error: 'return' : void function cannot return a value\n",
        '1 error generated.\n']


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithCommentBeforeVersion(expect.ErrorMessage):
    """Tests -D works well with #version preceded by comments."""

    shader = FileShader(
        """// comment 1
        /*
         * comment 2
         */
        #version 450 core
        void main(){X}
        void foo(){Y}""", '.vert')
    glslc_args = ['-DX=', '-DY=return 3;', shader]

    expected_error = [
        shader, ":7: error: 'return' : void function cannot return a value\n",
        '1 error generated.\n']


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithCommentAfterVersion(expect.ErrorMessage):
    """Tests -D works well with #version followed by comments."""

    shader = FileShader(
        """

        #version 150 core /*
        comment
        */
        void main(){X}
        void foo(){Y}""", '.vert')
    glslc_args = ['-DX=', '-DY=return 3;', shader]

    expected_error = [
        shader, ":7: error: 'return' : void function cannot return a value\n",
        '1 error generated.\n']


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithDashStd(expect.ErrorMessageSubstr):
    """Tests -D works well with -std."""

    shader = FileShader('void main(){X}\nvoid foo(){Y}', '.vert')
    glslc_args = ['-DX=', '-DY=return 3;', '-std=310es', shader]

    expected_error_substr = [
        shader, ":2: error: 'return' : void function cannot return a value\n",
        '1 error generated.\n']


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithDashStdAndVersion(expect.ErrorMessage):
    """Tests -D works well with both -std and #version."""

    shader = FileShader(
        """#version 310 es
        void main(){X}
        void foo(){Y}""", '.vert')
    glslc_args = ['-DX=', '-DY=return 3;', '-std=450core', shader]

    expected_error = [
        shader, ': warning: (version, profile) forced to be (450, ',
        'core), while in source code it is (310, es)\n',
        shader, ":3: error: 'return' : void function cannot return a value\n",
        '1 warning and 1 error generated.\n']


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithDashStdAndVersionAndComments(expect.ErrorMessage):
    """Tests -D works well with -std, #version, and comments around it."""

    shader = FileShader(
        """// comment before

        #version 310 es /* comment after
        */

        void main(){X}
        void foo(){Y}""", '.vert')
    glslc_args = ['-DX=', '-DY=return 3;', '-std=450core', shader]

    expected_error = [
        shader, ': warning: (version, profile) forced to be (450, core), while '
        'in source code it is (310, es)\n',
        shader, ":7: error: 'return' : void function cannot return a value\n",
        '1 warning and 1 error generated.\n']


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithDashE(expect.ReturnCodeIsZero,
                            expect.StdoutMatch):
    """Tests -E outputs expanded -D macros."""

    shader = FileShader(
        """
        void main(){Y}
""", '.vert')
    glslc_args = ['-DY=return 3;', '-E', '-std=450core', shader]

    expected_stdout =  [
        """
        void main() { return 3; }
"""]

@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithDashEIfDef(expect.ReturnCodeIsZero,
                                 expect.StdoutMatch):
    """Tests -E processes -DX #ifdefs correctly."""

    shader = FileShader(
        """
        #ifdef X
          void f() { }
        #else
          void f() { int x; }
        #endif
        void main(){ return 3; }
""", '.vert')
    glslc_args = ['-DX', '-E', '-std=450core', shader]

    expected_stdout =  [
        """

          void f() { }



        void main() { return 3; }
"""]

@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithDashEIfNDef(expect.ReturnCodeIsZero,
                                  expect.StdoutMatch):
    """Tests -E processes -DX #ifndefs correctly."""

    shader = FileShader(
        """
        #ifndef X
          void f() { }
        #else
          void f() { int x; }
        #endif
        void main(){ return 3; }
""", '.vert')
    glslc_args = ['-DX', '-E', '-std=450core', shader]

    expected_stdout =  [
        """



          void f() { int x; }

        void main() { return 3; }
"""]


@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithDashEEqIfDef(expect.ReturnCodeIsZero,
                                   expect.StdoutMatch):
    """Tests -E processes -DX= #ifdefs correctly."""

    shader = FileShader(
        """
        #ifdef X
          void f() { }
        #else
          void f() { int x; }
        #endif
        void main() { return 3; }
""", '.vert')
    glslc_args = ['-DX=', '-E', '-std=450core', shader]

    expected_stdout =  [
        """

          void f() { }



        void main() { return 3; }
"""]

@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithDashEEqIfNDef(expect.ReturnCodeIsZero,
                                    expect.StdoutMatch):
    """Tests -E processes -DX= #ifndefs correctly."""

    shader = FileShader(
        """
        #ifndef X
          void f() { }
        #else
          void f() { int x; }
        #endif
        void main(){ return 3; }
""", '.vert')
    glslc_args = ['-DX=', '-E', '-std=450core', shader]

    expected_stdout =  [
        """



          void f() { int x; }

        void main() { return 3; }
"""]

@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithDashEFunctionMacro(expect.ReturnCodeIsZero,
                                         expect.StdoutMatch):
    """Tests -E processes -D function macros correctly."""

    shader = FileShader(
        """
        void main(){ return FOO(3); }
""", '.vert')
    glslc_args = ['-DFOO(x)=(2*x+1)*x*x', '-E', '-std=450core', shader]

    expected_stdout =  [
        """
        void main() { return(2 * 3 + 1) * 3 * 3; }
"""]

@inside_glslc_testsuite('OptionCapD')
class TestDashCapDWithDashENestedMacro(expect.ReturnCodeIsZero,
                                       expect.StdoutMatch):
    """Tests -E processes referencing -D options correctly."""

    shader = FileShader(
        """
        void main() { return X; }
""", '.vert')
    glslc_args = ['-DY=4', '-DX=Y', '-E', '-std=450core', shader]

    expected_stdout =  [
        """
        void main() { return 4; }
"""]



#!/usr/bin/env python3

# Copyright 2016 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Updates build-version.inc in the current directory, unless the update is
# identical to the existing content.
#
# Args: <shaderc-dir> <spirv-tools-dir> <glslang-dir>
#
# For each directory, there will be a line in build-version.inc containing that
# directory's "git describe" output enclosed in double quotes and appropriately
# escaped.

import datetime
import errno
import os.path
import re
import subprocess
import sys
import time

def mkdir_p(directory):
    """Make the directory, and all its ancestors as required.  Any of the
    directories are allowed to already exist."""

    if directory == "":
        # We're being asked to make the current directory.
        return

    try:
        os.makedirs(directory)
    except OSError as e:
        if e.errno == errno.EEXIST and os.path.isdir(directory):
            pass
        else:
            raise

def command_output(cmd, directory):
    """Runs a command in a directory and returns its standard output stream.

    Captures the standard error stream.

    Raises a RuntimeError if the command fails to launch or otherwise fails.
    """
    p = subprocess.Popen(cmd,
                         cwd=directory,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    (stdout, _) = p.communicate()
    if p.returncode != 0:
        raise RuntimeError('Failed to run {} in {}'.format(cmd, directory))
    return stdout


def deduce_software_version(directory):
    """Returns a software version number parsed from the CHANGES file in the
    given directory.

    The CHANGES file describes most recent versions first.
    """

    # Match the first well-formed version-and-date line.
    # It must be on the third line, and it must have an ISO date.
    # Allow trailing whitespace in the checked-out source code has
    # unexpected carriage returns on a linefeed-only system such as
    # Linux.
    pattern = re.compile(r'^(v\d+\.\d+(-dev|[\.-]rc\d+)?) \d\d\d\d-\d\d-\d\d\s*$')
    changes_file = os.path.join(directory, 'CHANGES')
    with open(changes_file, errors='replace') as f:
        lines = f.readlines()[2:]
        if len(lines) > 0:
            match = pattern.match(lines[0])
            if match:
                return match.group(1)
    raise Exception('No version number found in {}'.format(changes_file))


def describe(directory):
    """Returns a string describing the current Git HEAD version as
    descriptively as possible.

    Runs 'git describe', or alternately 'git rev-parse HEAD', in
    directory.  If successful, returns the output; otherwise returns
    'unknown hash, <date>'.
    """
    try:
        # decode() is needed here for Python3 compatibility. In Python2,
        # str and bytes are the same type, but not in Python3.
        # Popen.communicate() returns a bytes instance, which needs to be
        # decoded into text data first in Python3. And this decode() won't
        # hurt Python2.
        return command_output(['git', 'describe'], directory).rstrip().decode()
    except:
        try:
            return command_output(
                ['git', 'rev-parse', 'HEAD'], directory).rstrip().decode()
        except:
            # This is the fallback case where git gives us no information,
            # e.g. because the source tree might not be in a git tree.
            # In this case, usually use a timestamp.  However, to ensure
            # reproducible builds, allow the builder to override the wall
            # clock time with enviornment variable SOURCE_DATE_EPOCH
            # containing a (presumably) fixed timestamp.
            timestamp = int(os.environ.get('SOURCE_DATE_EPOCH', time.time()))
            formatted = datetime.date.fromtimestamp(timestamp).isoformat()
            return 'unknown hash, {}'.format(formatted)


def get_version_string(project, directory):
    """Returns a detailed version string for a given project with its
    directory, which consists of software version string and git description
    string."""
    detailed_version_string_lst = [project]
    if project != 'glslang':
        detailed_version_string_lst.append(deduce_software_version(directory))
    detailed_version_string_lst.append(describe(directory).replace('"', '\\"'))
    return ' '.join(detailed_version_string_lst)


def main():
    if len(sys.argv) != 5:
        print(('usage: {} <shaderc-dir> <spirv-tools-dir> <glslang-dir> <output-file>'.format(
            sys.argv[0])))
        sys.exit(1)

    projects = ['shaderc', 'spirv-tools', 'glslang']
    new_content = ''.join([
        '"{}\\n"\n'.format(get_version_string(p, d))
        for (p, d) in zip(projects, sys.argv[1:])
    ])

    output_file = sys.argv[4]
    mkdir_p(os.path.dirname(output_file))

    if os.path.isfile(output_file):
        with open(output_file, 'r') as f:
            if new_content == f.read():
                return
    with open(output_file, 'w') as f:
        f.write(new_content)


if __name__ == '__main__':
    main()



# Copyright 2015 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import SpecializedString
from environment import File, Directory


@inside_glslc_testsuite('Include')
class VerifyIncludeOneSibling(expect.StdoutMatch):
    """Tests #including a sibling file."""

    environment = Directory('.', [
        File('a.vert', '#version 140\ncontent a\n#include "b"\n'),
        File('b', 'content b\n')])

    glslc_args = ['-E', 'a.vert']

    expected_stdout = \
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "a.vert"

content a
#line 0 "b"
 content b
#line 3 "a.vert"

"""

@inside_glslc_testsuite('Include')
class VerifyIncludeNotFound(expect.ErrorMessage):
    """Tests #including a not existing sibling file."""

    environment = Directory('.', [
        File('a.vert', '#version 140\ncontent a\n#include "b"\n')])

    glslc_args = ['-E', 'a.vert']
    expected_error = [
        "a.vert:3: error: '#include' : Cannot find or open include file. for header name: b\n",
        '1 error generated.\n'
    ]

@inside_glslc_testsuite('Include')
class VerifyCompileIncludeOneSibling(expect.ValidObjectFile):
    """Tests #including a sibling file via full compilation."""

    environment = Directory('.', [
        File('a.vert', '#version 140\nvoid foo(){}\n#include "b"\n'),
        File('b', 'void main(){foo();}\n')])

    glslc_args = ['a.vert']

@inside_glslc_testsuite('Include')
class VerifyIncludeWithoutNewline(expect.ErrorMessageSubstr):
    """Tests a #include without a newline."""

    environment = Directory('.', [
        File('a.vert', '#version 140\n#include "b"'),
        File('b', 'content b\n')])

    glslc_args = ['-E', 'a.vert']

    expected_error_substr = 'expected newline after header name: b'


@inside_glslc_testsuite('Include')
class VerifyCompileIncludeWithoutNewline(expect.ValidObjectFile):
    """Tests a #include without a newline via full compilation."""

    environment = Directory('.', [
        File('a.vert',
             """#version 140
             void main
             #include "b"
             """),
        File('b',
             """#define PAR ()
             PAR{}
             """)])

    glslc_args = ['a.vert']

@inside_glslc_testsuite('Include')
class VerifyIncludeTwoSiblings(expect.StdoutMatch):
    """Tests #including two sibling files."""

    environment = Directory('.', [
        File('b.vert', '#version 140\n#include "a"\ncontent b\n#include "c"\n'),
        File('a', 'content a\n'),
        File('c', 'content c\n')])

    glslc_args = ['-E', 'b.vert']

    expected_stdout = \
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "b.vert"

#line 0 "a"
content a
#line 2 "b.vert"
 content b
#line 0 "c"
 content c
#line 4 "b.vert"

"""

@inside_glslc_testsuite('Include')
class VerifyCompileIncludeTwoSiblings(expect.ValidObjectFile):
    """Tests #including two sibling files via full compilation."""

    environment = Directory('.', [
        File('b.vert',
             """#version 140
             #include "a"
             void bfun(){afun();}
             #include "c"
             """),
        File('a',
             """void afun(){}
             #define BODY {}
             """),
        File('c', 'void main() BODY\n')])

    glslc_args = ['b.vert']

@inside_glslc_testsuite('Include')
class VerifyNestedIncludeAmongSiblings(expect.StdoutMatch):
    """Tests #include inside #included sibling files."""

    environment = Directory('.', [
        File('a.vert', '#version 140\n#include "b"\ncontent a\n'),
        File('b', 'content b\n#include "c"\n'),
        File('c', 'content c\n')])

    glslc_args = ['-E', 'a.vert']

    expected_stdout = \
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "a.vert"

#line 0 "b"
content b
#line 0 "c"
 content c
#line 2 "b"
#line 2 "a.vert"
 content a
"""

@inside_glslc_testsuite('Include')
class VerifyCompileNestedIncludeAmongSiblings(expect.ValidObjectFile):
    """Tests #include inside #included sibling files via full compilation."""

    environment = Directory('.', [
        File('a.vert',
             """#version 140
             #define BODY {}
             #include "b"
             void main(){cfun();}
             """),
        File('b',
             """void bfun() BODY
             #include "c"
             """),
        File('c',
             """#define BF bfun()
             void cfun(){BF;}
             """)])

    glslc_args = ['a.vert']

@inside_glslc_testsuite('Include')
class VerifyIncludeSubdir(expect.StdoutMatch):
    """Tests #including a file from a subdirectory."""

    environment = Directory('.', [
        File('a.vert', '#version 140\ncontent a1\n#include "subdir/a"\ncontent a2\n'),
        Directory('subdir', [File('a', 'content suba\n')])])

    glslc_args = ['-E', 'a.vert']

    expected_stdout = \
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "a.vert"

content a1
#line 0 "subdir/a"
 content suba
#line 3 "a.vert"
 content a2
"""

@inside_glslc_testsuite('Include')
class VerifyCompileIncludeSubdir(expect.ValidObjectFile):
    """Tests #including a file from a subdirectory via full compilation."""

    environment = Directory('.', [
        File('a.vert',
             """#version 140
             #define BODY {}
             #include "subdir/a"
             void afun()BODY
             """),
        Directory('subdir', [File('a', 'void main() BODY\n')])])

    glslc_args = ['a.vert']

@inside_glslc_testsuite('Include')
class VerifyIncludeDeepSubdir(expect.StdoutMatch):
    """Tests #including a file from a subdirectory nested a few levels down."""

    environment = Directory('.', [
        File('a.vert',
             '#version 140\ncontent a1\n#include "dir/subdir/subsubdir/a"\ncontent a2\n'),
        Directory('dir', [
            Directory('subdir', [
                Directory('subsubdir', [File('a', 'content incl\n')])])])])

    glslc_args = ['-E', 'a.vert']

    expected_stdout = \
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "a.vert"

content a1
#line 0 "dir/subdir/subsubdir/a"
 content incl
#line 3 "a.vert"
 content a2
"""

@inside_glslc_testsuite('Include')
class VerifyCompileIncludeDeepSubdir(expect.ValidObjectFile):
    """Tests #including a file from a subdirectory nested a few levels down
       via full compilation."""

    environment = Directory('.', [
        File('a.vert',
             """#version 140
             #define BODY {}
             #include "dir/subdir/subsubdir/a"
             void afun()BODY
             """),
        Directory('dir', [
            Directory('subdir', [
                Directory('subsubdir', [File('a', 'void main() BODY\n')])])])])

    glslc_args = ['a.vert']


@inside_glslc_testsuite('Include')
class TestWrongPoundVersionInIncludingFile(expect.ValidObjectFileWithWarning):
    """Tests that warning message for #version directive in the including file
    has the correct filename."""

    environment = Directory('.', [
        File('a.vert', '#version 100000000\n#include "b.glsl"\n'),
        File('b.glsl', 'void main() {}\n')])
    glslc_args = ['-c', '-std=400', 'a.vert']

    expected_warning = [
        'a.vert: warning: (version, profile) forced to be (400, none),'
        ' while in source code it is (100000000, none)\n'
        '1 warning generated.\n'
    ]


# TODO(antiagainst): now #version in included files results in an error.
# Fix this after #version in included files are supported.
# TODO(dneto): I'm not sure what the expected result should be.
@inside_glslc_testsuite('Include')
class TestWrongPoundVersionInIncludedFile(expect.ErrorMessage):
    """Tests that warning message for #version directive in the included file
    has the correct filename."""

    environment = Directory('.', [
        File('a.vert', '#version 140\n#include "b.glsl"\nvoid main() {}'),
        File('b.glsl', '#version 10000000\n')])
    glslc_args = ['-E', 'a.vert']

    expected_error = [
        "b.glsl:1: error: '#version' : must occur first in shader\n",
        '1 error generated.\n'
    ]


@inside_glslc_testsuite('Include')
class VerifyRelativeInclude(expect.StdoutMatch):
    """Tests #including a relative sibling."""

    environment = Directory('.', [
        File('a.vert', '#version 140\ncontent a\n#include "foo/b.glsl"\n'),
        Directory('foo', [
            File('b.glsl', '#include "c.glsl"\ncontent b\n'),
            File('c.glsl', 'content c\n')
        ])])

    glslc_args = ['-E', 'a.vert']

    expected_stdout = \
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "a.vert"

content a
#line 0 "foo/b.glsl"
#line 0 "foo/c.glsl"
 content c
#line 1 "foo/b.glsl"
 content b
#line 3 "a.vert"

"""

@inside_glslc_testsuite('Include')
class VerifyNestedRelativeInclude(expect.StdoutMatch):
    """Tests #including a relative child file."""

    environment = Directory('.', [
        File('a.vert', '#version 140\ncontent a\n#include "foo/b.glsl"\n'),
        Directory('foo', [
            File('b.glsl', '#include "bar/c.glsl"\ncontent b\n'),
            Directory('bar', [
                File('c.glsl', 'content c\n')
            ])
        ])
    ])

    glslc_args = ['-E', 'a.vert']

    expected_stdout = \
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "a.vert"

content a
#line 0 "foo/b.glsl"
#line 0 "foo/bar/c.glsl"
 content c
#line 1 "foo/b.glsl"
 content b
#line 3 "a.vert"

"""


@inside_glslc_testsuite('Include')
class VerifyRelativeIncludeWithDashI(expect.StdoutMatch):
    """Tests #including a relative file from a -I directory."""

    environment = Directory('.', [
        File('a.vert', '#version 140\ncontent a\n#include "bar/b.glsl"\n'),
        Directory('foo', [
            Directory('bar', [
                File('b.glsl', '#include "c.glsl"\ncontent b\n'),
            ]),
            File('c.glsl', 'content c\n')
        ])
    ])

    glslc_args = ['-E', 'a.vert', '-Ifoo']

    expected_stdout = \
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "a.vert"

content a
#line 0 "foo/bar/b.glsl"
#line 0 "foo/c.glsl"
 content c
#line 1 "foo/bar/b.glsl"
 content b
#line 3 "a.vert"

"""


@inside_glslc_testsuite('Include')
class VerifyRelativeOverridesDashI(expect.StdoutMatch):
    """Tests that relative includes override -I parameters."""

    environment = Directory('.', [
        File('a.vert', '#version 140\ncontent a\n#include "b.glsl"\n'),
        File('b.glsl', 'content base_b\n'),
        Directory('foo', [
            File('b.glsl', '#include "c.glsl"\ncontent b\n'),
            File('c.glsl', 'content c\n')
        ])
    ])

    glslc_args = ['-E', 'a.vert', '-Ifoo']

    expected_stdout = \
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "a.vert"

content a
#line 0 "b.glsl"
 content base_b
#line 3 "a.vert"

"""


@inside_glslc_testsuite('Include')
class VerifyRelativeParent(expect.StdoutMatch):
    """Tests #including a parent file."""

    environment = Directory('.', [
        File('a.vert', '#version 140\ncontent a\n#include "b.glsl"\n'),
        File('c.glsl', 'content c\n'),
        Directory('foo', [
            File('b.glsl', '#include "../c.glsl"\ncontent b\n')
        ])
    ])

    glslc_args = ['-E', 'a.vert', '-Ifoo']

    expected_stdout = \
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "a.vert"

content a
#line 0 "foo/b.glsl"
#line 0 "foo/../c.glsl"
 content c
#line 1 "foo/b.glsl"
 content b
#line 3 "a.vert"

"""


@inside_glslc_testsuite('Include')
class VerifyRelativeNeighbourDirectory(expect.StdoutMatch):
    """Tests #including a relative file in a neighbour directory."""

    environment = Directory('.', [
        File('a.vert', '#version 140\ncontent a\n#include "foo/b.glsl"\n'),
        Directory('foo', [
            File('b.glsl', '#include "../bar/c.glsl"\ncontent b\n')
        ]),
        Directory('bar', [
            File('c.glsl', 'content c\n')
        ])
    ])

    glslc_args = ['-E', 'a.vert']

    expected_stdout = \
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "a.vert"

content a
#line 0 "foo/b.glsl"
#line 0 "foo/../bar/c.glsl"
 content c
#line 1 "foo/b.glsl"
 content b
#line 3 "a.vert"

"""


@inside_glslc_testsuite('Include')
class VerifyRelativeOnlyToSelf(expect.ErrorMessage):
    """Tests that a relative includes are only relative to yourself."""

    environment = Directory('.', [
        File('a.vert', '#version 140\ncontent a\n#include "foo/b.glsl"\n'),
        File('c.glsl', 'content c\n'),
        Directory('foo', [
            File('b.glsl', '#include "c.glsl"\ncontent b\n')
        ]),
    ])

    glslc_args = ['-E', 'a.vert']

    expected_error = [
        "foo/b.glsl:1: error: '#include' : "
        'Cannot find or open include file. for header name: c.glsl\n',
        '1 error generated.\n'
    ]


@inside_glslc_testsuite('Include')
class VerifyRelativeFromAbsolutePath(expect.StdoutMatch):
    """Tests that absolute files can relatively include."""

    environment = Directory('.', [
        File('a.vert', '#version 140\ncontent a\n#include "b.glsl"\n'),
        File('b.glsl', 'content b\n')
    ])

    glslc_args = ['-E', SpecializedString('$directory/a.vert')]

    expected_stdout = SpecializedString(
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "$directory/a.vert"

content a
#line 0 "$directory/b.glsl"
 content b
#line 3 "$directory/a.vert"

""")


@inside_glslc_testsuite('Include')
class VerifyDashIAbsolutePath(expect.StdoutMatch):
    """Tests that -I parameters can be absolute paths."""

    environment = Directory('.', [
        File('a.vert', '#version 140\ncontent a\n#include "b.glsl"\n'),
        Directory('foo', {
            File('b.glsl', 'content b\n')
        })
    ])

    glslc_args = ['-E', 'a.vert', '-I', SpecializedString('$directory/foo')]

    expected_stdout = SpecializedString(
"""#version 140
#extension GL_GOOGLE_include_directive : enable
#line 0 "a.vert"

content a
#line 0 "$directory/foo/b.glsl"
 content b
#line 3 "a.vert"

""")



# Copyright 2015 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
import os.path
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader, TempFileName
from builtins import bytes


@inside_glslc_testsuite('OptionDashO')
class TestOptionDashOConcatenatedArg(expect.SuccessfulReturn,
                                     expect.CorrectObjectFilePreamble):
    """Tests that we can concatenate -o and the output filename."""

    shader = FileShader('#version 140\nvoid main() {}', '.vert')
    glslc_args = ['-ofoo', shader]

    def check_output_foo(self, status):
        output_name = os.path.join(status.directory, 'foo')
        return self.verify_object_file_preamble(output_name)


@inside_glslc_testsuite('OptionDashO')
class ManyOutputFilesWithDashO(expect.ErrorMessage):
    """Tests -o and -c with several files generates an error."""

    shader1 = FileShader('', '.vert')
    shader2 = FileShader('', '.frag')
    glslc_args = ['-o', 'foo', '-c', shader1, shader2]
    expected_error = [
        'glslc: error: cannot specify -o when '
        'generating multiple output files\n']


@inside_glslc_testsuite('OptionDashO')
class OutputFileLocation(expect.SuccessfulReturn,
                         expect.CorrectObjectFilePreamble):
    """Tests that the -o flag puts a file in a new location."""

    shader = FileShader('#version 310 es\nvoid main() {}', '.frag')
    glslc_args = [shader, '-o', TempFileName('a.out')]

    def check_output_a_out(self, status):
        output_name = os.path.join(status.directory, 'a.out')
        return self.verify_object_file_preamble(output_name)


@inside_glslc_testsuite('OptionDashO')
class DashOMissingArgumentIsAnError(expect.ErrorMessage):
    """Tests that -o without an argument is an error."""

    glslc_args = ['-o']
    expected_error = ['glslc: error: argument to \'-o\' is missing ' +
                      '(expected 1 value)\n']


@inside_glslc_testsuite('OptionDashO')
class OutputFileBinaryAvoidsCRLFTranslation(expect.ReturnCodeIsZero,
                                            expect.NoOutputOnStderr,
                                            expect.NoGeneratedFiles,
                                            expect.CorrectBinaryLengthAndPreamble):
    """Tests that the -o flag emits a binary file without CR/LF translation.
    """

    # A shader whose compiled output has three bytes that are newlines.
    # If the output stream converts the newlines to CR/LF, then we end up
    # with a file that is 4k + 3 bytes long.  That will be caught by the
    # object file checks.
    SHADER_WITH_THREE_NEWLINES_IN_BINARY = """#version 450
       layout(location = 0) out uint ovar;
       void main() { ovar = 9; }
    """

    shader = FileShader(SHADER_WITH_THREE_NEWLINES_IN_BINARY, '.vert')
    glslc_args = [shader, '-o', '-']

    def check_stdout_binary(self, status):
        binary = bytes(status.stdout)
        newlines = [x for x in binary if x == ord('\n')]
        num_newlines = len(newlines)
        if num_newlines % 4 == 0:
            return False, "Bad test. Need nontrivial number of newlines"
        if num_newlines != 2:
            return False, ("Update this test. Expected 3 newlines in the "
                           "binary, but found {}").format(num_newlines)
        return self.verify_binary_length_and_header(bytes(status.stdout))



# Copyright 2015 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader


def empty_es_310_shader():
    return '#version 310 es\n void main() {}\n'


@inside_glslc_testsuite('OptionC')
class TestSingleDashCSingleFile(expect.ValidObjectFile):
    """Tests that glslc accepts -c [filename]."""

    shader = FileShader(empty_es_310_shader(), '.vert')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('OptionC')
class TestSingleFileSingleDashC(expect.ValidObjectFile):
    """Tests that glslc accepts [filename] -c."""

    shader = FileShader(empty_es_310_shader(), '.vert')
    glslc_args = [shader, '-c']


@inside_glslc_testsuite('OptionC')
class TestMultipleFiles(expect.ValidObjectFile):
    """Tests that glslc accepts -c and multiple source files."""

    shader1 = FileShader(empty_es_310_shader(), '.vert')
    shader2 = FileShader(empty_es_310_shader(), '.frag')
    glslc_args = ['-c', shader1, shader2]


@inside_glslc_testsuite('OptionC')
class TestMultipleDashC(expect.ValidObjectFile):
    """Tests that glslc accepts multiple -c and treated them as one."""

    shader1 = FileShader(empty_es_310_shader(), '.vert')
    shader2 = FileShader(empty_es_310_shader(), '.vert')
    glslc_args = ['-c', shader1, '-c', '-c', shader2]



# Copyright 2015 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader, StdinShader


@inside_glslc_testsuite('OptionCapE')
class TestDashCapENoDefs(expect.StdoutMatch):
    """Tests -E without any defines."""

    shader = FileShader('#version 140\nvoid main(){}', '.vert')
    expected_stdout = '#version 140\nvoid main() { }\n'
    glslc_args = ['-E', shader]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEGlslFileAccepted(expect.StdoutMatch):
    """Tests -E if we provide a .glsl file without an explicit stage."""

    shader = FileShader('#version 140\nvoid main(){}', '.glsl')
    expected_stdout = '#version 140\nvoid main() { }\n'
    glslc_args = ['-E', shader]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapESingleDefine(expect.StdoutMatch):
    """Tests -E with command-line define."""

    shader = FileShader('#version 140\nvoid main(){ int a = X; }', '.vert')
    expected_stdout = '#version 140\nvoid main() { int a = 4; }\n'
    glslc_args = ['-DX=4', '-E', shader]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEExpansion(expect.StdoutMatch):
    """Tests -E with macro expansion."""

    shader = FileShader('''#version 140
#define X 4
void main() {
  int a = X;
}
''', '.vert')

    expected_stdout = '''#version 140

void main() {
  int a = 4;
}
'''
    glslc_args = ['-E', shader]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEFunctionMacro(expect.StdoutMatch):
    """Tests -E with function-style macro expansion."""

    shader = FileShader('''#version 140
#define X(A) 4+A
void main() {
  int a = X(1);
}
''', '.vert')

    expected_stdout = '''#version 140

void main() {
  int a = 4 + 1;
}
'''
    glslc_args = ['-E', shader]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEPragma(expect.StdoutMatch):
    """Tests -E to make sure pragmas get retained."""

    shader = FileShader('''#version 140
#pragma optimize(off)
void main() {
}
''', '.vert')

    expected_stdout = '''#version 140
#pragma optimize(off)
void main() {
}
'''
    glslc_args = ['-E', shader]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEExtension(expect.StdoutMatch):
    """Tests -E to make sure extensions get retained."""

    shader = FileShader('''#version 140
#extension foo: require
void main() {
}
''', '.vert')

    expected_stdout = '''#version 140
#extension foo : require
void main() {
}
'''
    glslc_args = ['-E', shader]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapELine(expect.StdoutMatch):
    """Tests -E to make sure line numbers get retained."""

    shader = FileShader('''#version 140
#define X 4
#line X

#line 2 3

void main() {
}
''', '.vert')

    expected_stdout = '''#version 140

#line 4

#line 2 3

void main() {
}
'''
    glslc_args = ['-E', shader]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEError(expect.ErrorMessage):
    """Tests -E to make sure #errors get retained."""

    shader = FileShader('''#version 140
#if 1
  #error This is an error
#endif

void main() {
}
''', '.vert')

    expected_error = [
        shader, ':3: error: \'#error\' : This is an error\n',
        '1 error generated.\n']

    glslc_args = ['-E', shader]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEStdin(expect.StdoutMatch):
    """Tests to make sure -E works with stdin."""

    shader = StdinShader('''#version 140
void main() {
}
''')

    expected_stdout = '''#version 140
void main() {
}
'''
    glslc_args = ['-E', '-fshader-stage=vertex', shader]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEStdinDoesNotRequireShaderStage(expect.StdoutMatch):
    """Tests to make sure -E works with stdin even when no shader-stage
    is specified."""

    shader = StdinShader('''#version 140
void main() {
}
''')

    expected_stdout = '''#version 140
void main() {
}
'''
    glslc_args = ['-E', shader]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEMultipleFiles(expect.StdoutMatch):
    """Tests to make sure -E works with multiple files."""

    shader = StdinShader('''#version 140
void main() {
}
''')
    shader2 = FileShader('''#version 140
void function() {
}
''', '.vert')
    expected_stdout = '''#version 140
void main() {
}
#version 140
void function() {
}
'''
    glslc_args = ['-E', '-fshader-stage=vertex', shader, shader2]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEMultipleFilesWithoutStage(expect.StdoutMatch):
    """Tests to make sure -E works with multiple files even if we do not
    specify a stage."""

    shader = StdinShader('''#version 140
void main() {
}
''')
    shader2 = FileShader('''#version 140
void function() {
}
''', '.glsl')
    expected_stdout = '''#version 140
void main() {
}
#version 140
void function() {
}
'''
    glslc_args = ['-E', shader, shader2]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEOutputFile(expect.SuccessfulReturn, expect.ValidFileContents):
    """Tests to make sure -E works with output files."""

    shader = FileShader('''#version 140
void function() {
}
''', '.vert')
    expected_file_contents = '''#version 140
void function() {
}
'''
    target_filename = 'foo'
    glslc_args = ['-E', shader, '-ofoo']


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEWithS(expect.StdoutMatch):
    """Tests -E in the presence of -S."""

    shader = FileShader('#version 140\nvoid main(){}', '.vert')
    expected_stdout = '#version 140\nvoid main() { }\n'
    glslc_args = ['-E', '-S', shader]


@inside_glslc_testsuite('OptionCapE')
class TestMultipileDashCapE(expect.StdoutMatch):
    """Tests that using -E multiple times works."""

    shader = FileShader('#version 140\nvoid main(){}', '.vert')
    expected_stdout = '#version 140\nvoid main() { }\n'
    glslc_args = ['-E', '-E', shader, '-E']


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEAfterFile(expect.StdoutMatch):
    """Tests that using -E after the filename also works."""

    shader = FileShader('#version 140\nvoid main(){}', '.vert')
    expected_stdout = '#version 140\nvoid main() { }\n'
    glslc_args = [shader, '-E']


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEWithDashC(expect.StdoutMatch):
    """Tests to make sure -E works in the presence of -c."""

    shader = FileShader('''#version 140
void main() {
}
''', '.vert')
    shader2 = FileShader('''#version 140
void function() {
}
''', '.vert')
    expected_stdout = '''#version 140
void main() {
}
#version 140
void function() {
}
'''
    glslc_args = ['-E', '-c', shader, shader2]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEWithPPErrors(expect.ErrorMessage):
    """Tests to make sure -E outputs error messages for preprocessing errors."""

    shader = FileShader('''#version 310 es
#extension s enable // missing :
#defin A // Bad define
#if X // In glsl X must be defined for X to work.
      // Lack of endif.
void main() {
}
''', '.vert')
    expected_error = [
        shader, ':2: error: \'#extension\' : \':\' missing after extension',
        ' name\n',
        shader, ':3: error: \'#\' : invalid directive: defin\n',
        shader, ':4: error: \'preprocessor evaluation\' : undefined macro in',
        ' expression not allowed in es profile X\n',
        shader, ':8: error: \'\' : missing #endif\n',
        '4 errors generated.\n']
    glslc_args = ['-E', shader]


@inside_glslc_testsuite('OptionCapE')
class TestDashCapEStdinErrors(expect.ErrorMessage):
    """Tests that -E outputs error messages correctly for stdin input."""

    shader = StdinShader('''#version 310 es
#extension s enable // missing :
void main() {
}
''')
    expected_error = [
        '<stdin>:2: error: \'#extension\' : \':\' missing after extension',
        ' name\n',
        '1 error generated.\n']
    glslc_args = ['-E', shader]



#!/usr/bin/env python3

# Copyright 2015 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Removes all files with a certain suffix in a given path recursively.

# Arguments: path suffix

import os
import sys


def main():
    path = sys.argv[1]
    suffix = sys.argv[2]
    for root, _, filenames in os.walk(path):
        for filename in filenames:
            if filename.endswith(suffix):
                os.remove(os.path.join(root, filename))


if __name__ == '__main__':
    main()



# Copyright 2018 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader

HLSL_SHADER_WITH_HALF_TYPE = """
float4 main() : SV_Target0 {
  half h0 = (half)0.0;
  half h1 = (half)1.0;
  half h2 = (half)2.0;
  half h3 = (half)3.0;
  half4 v = (half4)(h0,h1,h2,h3) * (half)2.0;
  return (float4)(v);
}
"""


@inside_glslc_testsuite('OptionFHlsl16BitTypes')
class TestHlsl16BitTypes_EnablesCapability(expect.ValidAssemblyFileWithSubstr):
    """Tests that -fhlsl_16bit_types enables the 16bit floating point capability."""

    shader = FileShader(HLSL_SHADER_WITH_HALF_TYPE, '.frag')
    glslc_args = ['-S', '-x', 'hlsl', '-fhlsl-16bit-types', '-fauto-bind-uniforms', shader]
    expected_assembly_substr = 'OpCapability Float16';


@inside_glslc_testsuite('OptionFHlsl16BitTypes')
class TestHlsl16BitTypes_CreatesType(expect.ValidAssemblyFileWithSubstr):
    """Tests that -fhlsl_16bit_types creates the 16bit floating point capability."""

    shader = FileShader(HLSL_SHADER_WITH_HALF_TYPE, '.frag')
    glslc_args = ['-S', '-x', 'hlsl', '-fhlsl-16bit-types', '-fauto-bind-uniforms', shader]
    expected_assembly_substr = '= OpTypeFloat 16';



# Copyright 2019 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader

# A GLSL shader using the clamp, max, and min builtin functions.
GLSL_FRAG_SHADER_WITH_CLAMP = """#version 450
layout(location=0) in vec4 i;
layout(location=0) out vec4 o;
void main() {
  o = clamp(i, vec4(0.5), vec4(1.0))
  +   max(i, vec4(0.5))
  +   min(i, vec4(0.5));
}
"""


@inside_glslc_testsuite('OptionFNanClamp')
class TestClampMapsToFClampByDefault(expect.ValidAssemblyFileWithSubstr):
    shader = FileShader(GLSL_FRAG_SHADER_WITH_CLAMP, '.frag')
    glslc_args = ['-S', shader]
    expected_assembly_substr = 'OpExtInst %v4float %1 FClamp'


@inside_glslc_testsuite('OptionFNanClamp')
class TestMaxMapsToFMaxByDefault(expect.ValidAssemblyFileWithSubstr):
    shader = FileShader(GLSL_FRAG_SHADER_WITH_CLAMP, '.frag')
    glslc_args = ['-S', shader]
    expected_assembly_substr = 'OpExtInst %v4float %1 FMax'


@inside_glslc_testsuite('OptionFNanClamp')
class TestMinMapsToFMinByDefault(expect.ValidAssemblyFileWithSubstr):
    shader = FileShader(GLSL_FRAG_SHADER_WITH_CLAMP, '.frag')
    glslc_args = ['-S', shader]
    expected_assembly_substr = 'OpExtInst %v4float %1 FMin'


@inside_glslc_testsuite('OptionFNanClamp')
class TestClampMapsToNClampWithFlag(expect.ValidAssemblyFileWithSubstr):
    shader = FileShader(GLSL_FRAG_SHADER_WITH_CLAMP, '.frag')
    glslc_args = ['-S', '-fnan-clamp', shader]
    expected_assembly_substr = 'OpExtInst %v4float %1 NClamp'

@inside_glslc_testsuite('OptionFNanClamp')
class TestMaxMapsToNMaxWithFlag(expect.ValidAssemblyFileWithSubstr):
    shader = FileShader(GLSL_FRAG_SHADER_WITH_CLAMP, '.frag')
    glslc_args = ['-S', '-fnan-clamp', shader]
    expected_assembly_substr = 'OpExtInst %v4float %1 NMax'


@inside_glslc_testsuite('OptionFNanClamp')
class TestMinMapsToNMinWithFlag(expect.ValidAssemblyFileWithSubstr):
    shader = FileShader(GLSL_FRAG_SHADER_WITH_CLAMP, '.frag')
    glslc_args = ['-S', '-fnan-clamp', shader]
    expected_assembly_substr = 'OpExtInst %v4float %1 NMin'



# Copyright 2015 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader, StdinShader


@inside_glslc_testsuite('StdInOut')
class VerifyStdinWorks(expect.ValidObjectFile):
    """Tests glslc accepts vertex shader extension (.vert)."""

    shader = StdinShader('#version 140\nvoid main() { }')
    glslc_args = ['-c', '-fshader-stage=vertex', shader]


@inside_glslc_testsuite('StdInOut')
class VerifyStdoutWorks(
    expect.ReturnCodeIsZero, expect.StdoutMatch, expect.StderrMatch):

    shader = FileShader('#version 140\nvoid main() {}', '.vert')
    glslc_args = [shader, '-o', '-']

    # We expect SOME stdout, we just do not care what.
    expected_stdout = True
    expected_stderr = ''



# Copyright 2016 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
import re

@inside_glslc_testsuite('OptionDashDashVersion')
class TestVersionContainsShaderc(expect.StdoutMatch, expect.ReturnCodeIsZero):
    """Tests --version output contains 'shaderc'."""
    glslc_args = ['--version']
    expected_stdout = re.compile('^shaderc')


@inside_glslc_testsuite('OptionDashDashVersion')
class TestVersionContainsSpirvTools(expect.StdoutMatch, expect.ReturnCodeIsZero):
    """Tests --version output contains 'spirv-tools'."""
    glslc_args = ['--version']
    expected_stdout = re.compile('\nspirv-tools')


@inside_glslc_testsuite('OptionDashDashVersion')
class TestVersionContainsGlslang(expect.StdoutMatch, expect.ReturnCodeIsZero):
    """Tests --version output contains 'glslang'."""
    glslc_args = ['--version']
    expected_stdout = re.compile('\nglslang')


@inside_glslc_testsuite('OptionDashDashVersion')
class TestVersionContainsTarget(expect.StdoutMatch, expect.ReturnCodeIsZero):
    """Tests --version output contains 'Target:'."""
    glslc_args = ['--version']
    expected_stdout = re.compile('\nTarget: SPIR-V \d+\.\d+')



# Copyright 2015 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from environment import Directory, File
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader, StdinShader

VERTEX_ONLY_SHADER_WITH_PRAGMA = \
    """#version 310 es
    #pragma shader_stage(vertex)
    void main() {
        gl_Position = vec4(1.);
    }"""

FRAGMENT_ONLY_SHADER_WITH_PRAGMA = \
    """#version 310 es
    #pragma shader_stage(fragment)
    void main() {
        gl_FragDepth = 10.;
    }"""


TESS_CONTROL_ONLY_SHADER_WITH_PRAGMA = \
    """#version 440 core
    #pragma shader_stage(tesscontrol)
    layout(vertices = 3) out;
    void main() { }"""


TESS_EVAL_ONLY_SHADER_WITH_PRAGMA = \
    """#version 440 core
    #pragma shader_stage(tesseval)
    layout(triangles) in;
    void main() { }"""


GEOMETRY_ONLY_SHDER_WITH_PRAGMA = \
    """#version 150 core
    #pragma shader_stage(geometry)
    layout (triangles) in;
    layout (line_strip, max_vertices = 4) out;
    void main() { }"""


COMPUTE_ONLY_SHADER_WITH_PRAGMA = \
    """#version 310 es
    #pragma shader_stage(compute)
    void main() {
        uvec3 temp = gl_WorkGroupID;
    }"""

# In the following tests,
# PSS stands for PragmaShaderStage, and OSS stands for OptionShaderStage.


@inside_glslc_testsuite('PragmaShaderStage')
class TestPSSWithGlslExtension(expect.ValidObjectFile):
    """Tests #pragma shader_stage() with .glsl extension."""

    shader = FileShader(VERTEX_ONLY_SHADER_WITH_PRAGMA, '.glsl')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('PragmaShaderStage')
class TestPSSWithUnkownExtension(expect.ValidObjectFile):
    """Tests #pragma shader_stage() with unknown extension."""

    shader = FileShader(VERTEX_ONLY_SHADER_WITH_PRAGMA, '.unkown')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('PragmaShaderStage')
class TestPSSWithStdin(expect.ValidObjectFile):
    """Tests #pragma shader_stage() with stdin."""

    shader = StdinShader(VERTEX_ONLY_SHADER_WITH_PRAGMA)
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('PragmaShaderStage')
class TestPSSWithSameShaderExtension(expect.ValidObjectFile):
    """Tests that #pragma shader_stage() specifies the same stage as file
    extesion."""

    shader = FileShader(VERTEX_ONLY_SHADER_WITH_PRAGMA, '.vert')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('PragmaShaderStage')
class TestPSSOverrideShaderExtension(expect.ValidObjectFile):
    """Tests that #pragma shader_stage() overrides file extension."""

    shader = FileShader(VERTEX_ONLY_SHADER_WITH_PRAGMA, '.frag')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('PragmaShaderStage')
class TestOSSOverridePSS(expect.ValidObjectFile):
    """Tests that -fshader-stage overrides #pragma shader_stage()."""

    # wrong pragma and wrong file extension
    shader = FileShader(
        """#version 310 es
        #pragma shader_stage(fragment)
        void main() {
            gl_Position = vec4(1.);
        }""", '.frag')
    # -fshader-stage to the rescue! ^.^
    glslc_args = ['-c', '-fshader-stage=vertex', shader]


@inside_glslc_testsuite('PragmaShaderStage')
class TestMultipleSamePSS(expect.ValidObjectFile):
    """Tests that multiple identical #pragma shader_stage() works."""

    shader = FileShader(
        """#version 310 es
        #pragma shader_stage(vertex)
        #pragma shader_stage(vertex)
        void main() {
        #pragma shader_stage(vertex)
            gl_Position = vec4(1.);
        #pragma shader_stage(vertex)
        }
        #pragma shader_stage(vertex)
        #pragma shader_stage(vertex)
        """, '.glsl')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('PragmaShaderStage')
class TestConflictingPSS(expect.ErrorMessage):
    """Conflicting #pragma shader_stage() directives result in an error."""

    shader = FileShader(
        """#version 310 es
        #pragma shader_stage(vertex)
        void main() {
            gl_Position = vec4(1.);
        }
        #pragma shader_stage(fragment)
        """, '.glsl')
    glslc_args = ['-c', shader]
    expected_error = [
        shader, ":6: error: '#pragma': conflicting stages for 'shader_stage' "
        "#pragma: 'fragment' (was 'vertex' at ", shader, ':2)\n']


@inside_glslc_testsuite('PragmaShaderStage')
class TestAllPSSValues(expect.ValidObjectFile):
    """Tests all possible #pragma shader_stage() values."""

    shader1 = FileShader(VERTEX_ONLY_SHADER_WITH_PRAGMA, '.glsl')
    shader2 = FileShader(FRAGMENT_ONLY_SHADER_WITH_PRAGMA, '.glsl')
    shader3 = FileShader(TESS_CONTROL_ONLY_SHADER_WITH_PRAGMA, '.glsl')
    shader4 = FileShader(TESS_EVAL_ONLY_SHADER_WITH_PRAGMA, '.glsl')
    shader5 = FileShader(GEOMETRY_ONLY_SHDER_WITH_PRAGMA, '.glsl')
    shader6 = FileShader(COMPUTE_ONLY_SHADER_WITH_PRAGMA, '.glsl')
    glslc_args = ['-c', shader1, shader2, shader3, shader4, shader5, shader6]


@inside_glslc_testsuite('PragmaShaderStage')
class TestWrongPSSValue(expect.ErrorMessage):
    """Tests that #pragma shader_stage([wrong-stage]) results in an error."""

    shader = FileShader(
        """#version 310 es
        #pragma shader_stage(superstage)
        void main() {
            gl_Position = vec4(1.);
        }
        """, '.glsl')
    glslc_args = ['-c', shader]
    expected_error = [
        shader, ":2: error: '#pragma': invalid stage for 'shader_stage' "
        "#pragma: 'superstage'\n"]


@inside_glslc_testsuite('PragmaShaderStage')
class TestEmptyPSSValue(expect.ErrorMessage):
    """Tests that #pragma shader_stage([empty]) results in an error."""

    shader = FileShader(
        """#version 310 es
        #pragma shader_stage()
        void main() {
            gl_Position = vec4(1.);
        }
        """, '.glsl')
    glslc_args = ['-c', shader]
    expected_error = [
        shader, ":2: error: '#pragma': invalid stage for 'shader_stage' "
        "#pragma: ''\n"]


@inside_glslc_testsuite('PragmaShaderStage')
class TestFirstPSSBeforeNonPPCode(expect.ErrorMessage):
    """Tests that the first #pragma shader_stage() should appear before
    any non-preprocessing code."""

    shader = FileShader(
        """#version 310 es
        #ifndef REMOVE_UNUSED_FUNCTION
          int inc(int i) { return i + 1; }
        #endif
        #pragma shader_stage(vertex)
        void main() {
            gl_Position = vec4(1.);
        }
        """, '.glsl')
    glslc_args = ['-c', shader]
    expected_error = [
        shader, ":5: error: '#pragma': the first 'shader_stage' #pragma "
        'must appear before any non-preprocessing code\n']


@inside_glslc_testsuite('PragmaShaderStage')
class TestPSSMultipleErrors(expect.ErrorMessage):
    """Tests that if there are multiple errors, they are all reported."""

    shader = FileShader(
        """#version 310 es
        #pragma shader_stage(idontknow)
        #pragma shader_stage(vertex)
        void main() {
            gl_Position = vec4(1.);
        }
        #pragma shader_stage(fragment)
        """, '.glsl')
    glslc_args = ['-c', shader]
    expected_error = [
        shader, ":2: error: '#pragma': invalid stage for 'shader_stage' "
        "#pragma: 'idontknow'\n",
        shader, ":3: error: '#pragma': conflicting stages for 'shader_stage' "
        "#pragma: 'vertex' (was 'idontknow' at ", shader, ':2)\n',
        shader, ":7: error: '#pragma': conflicting stages for 'shader_stage' "
        "#pragma: 'fragment' (was 'idontknow' at ", shader, ':2)\n']


@inside_glslc_testsuite('PragmaShaderStage')
class TestSpacesAroundPSS(expect.ValidObjectFile):
    """Tests that spaces around #pragma shader_stage() works."""

    shader = FileShader(
        """#version 310 es
               #     pragma      shader_stage     (    vertex     )
        void main() {
            gl_Position = vec4(1.);
        }
        """, '.glsl')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('PragmaShaderStage')
class TestTabsAroundPSS(expect.ValidObjectFile):
    """Tests that tabs around #pragma shader_stage() works."""

    shader = FileShader(
        """#version 310 es
        \t\t#\tpragma\t\t\tshader_stage\t\t(\t\t\t\tvertex\t\t)\t\t\t\t
        void main() {
            gl_Position = vec4(1.);
        }
        """, '.glsl')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('PragmaShaderStage')
class TestPSSWithMacro(expect.ValidObjectFile):
    """Tests that #pragma shader_stage() works with macros."""

    shader = FileShader(
        """#version 310 es
        #if 0
          some random stuff here which can cause a problem
        #else
        # pragma shader_stage(vertex)
        #endif
        void main() {
            gl_Position = vec4(1.);
        }
        """, '.glsl')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('PragmaShaderStage')
class TestPSSWithCmdLineMacroDef(expect.ValidObjectFile):
    """Tests that macro definitions passed in from command line work."""

    shader = FileShader(
        """#version 310 es
        #ifdef IS_A_VERTEX_SHADER
        # pragma shader_stage(vertex)
        #else
        # pragma shader_stage(fragment)
        #endif
        void main() {
            gl_Position = vec4(1.);
        }
        """, '.glsl')
    glslc_args = ['-c', '-DIS_A_VERTEX_SHADER', shader]


@inside_glslc_testsuite('PragmaShaderStage')
class TestNoMacroExpansionInsidePSS(expect.ErrorMessage):
    """Tests that there is no macro expansion inside #pragma shader_stage()."""

    shader = FileShader(
        """#version 310 es
        #pragma shader_stage(STAGE_FROM_CMDLINE)
        void main() {
            gl_Position = vec4(1.);
        }
        """, '.glsl')
    glslc_args = ['-c', '-DSTAGE_FROM_CMDLINE=vertex', shader]

    expected_error = [
        shader, ":2: error: '#pragma': invalid stage for 'shader_stage' "
        "#pragma: 'STAGE_FROM_CMDLINE'\n"]


@inside_glslc_testsuite('PragmaShaderStage')
class TestPSSWithPoundLine310(expect.ErrorMessage):
    """Tests that #pragma shader_stage() works with #line."""

    shader = FileShader(
        """#version 310 es
        #pragma shader_stage(unknown)
        #line 42
        #pragma shader_stage(google)
        #line 100
        #pragma shader_stage(elgoog)
        void main() {
            gl_Position = vec4(1.);
        }
        """, '.glsl')
    glslc_args = ['-c', shader]

    expected_error = [
        shader, ":2: error: '#pragma': invalid stage for 'shader_stage' "
        "#pragma: 'unknown'\n",
        shader, ":42: error: '#pragma': conflicting stages for 'shader_stage' "
        "#pragma: 'google' (was 'unknown' at ", shader, ':2)\n',
        shader, ":100: error: '#pragma': conflicting stages for 'shader_stage' "
        "#pragma: 'elgoog' (was 'unknown' at ", shader, ':2)\n']


@inside_glslc_testsuite('PragmaShaderStage')
class TestPSSWithPoundLine150(expect.ErrorMessage):
    """Tests that #pragma shader_stage() works with #line.

    For older desktop versions, a #line directive specify the line number of
    the #line directive, not the next line.
    """

    shader = FileShader(
        """#version 150
        #pragma shader_stage(unknown)
        #line 42
        #pragma shader_stage(google)
        #line 100
        #pragma shader_stage(elgoog)
        void main() {
            gl_Position = vec4(1.);
        }
        """, '.glsl')
    glslc_args = ['-c', shader]

    expected_error = [
        shader, ":2: error: '#pragma': invalid stage for 'shader_stage' "
        "#pragma: 'unknown'\n",
        shader, ":43: error: '#pragma': conflicting stages for 'shader_stage' "
        "#pragma: 'google' (was 'unknown' at ", shader, ':2)\n',
        shader, ":101: error: '#pragma': conflicting stages for 'shader_stage' "
        "#pragma: 'elgoog' (was 'unknown' at ", shader, ':2)\n']


@inside_glslc_testsuite('PragmaShaderStage')
class ErrorBeforePragma(expect.ErrorMessage):
    """Tests that errors before pragmas are emitted."""

    shader = FileShader(
        """#version 310 es
        #something
        #pragma shader_stage(vertex)
        void main() {
            gl_Position = vec4(1.);
        }
        """, '.glsl')
    glslc_args = ['-c', shader]
    expected_error = [shader, ':2: error: \'#\' : invalid directive:',
                      ' something\n'
                      '1 error generated.\n']


@inside_glslc_testsuite('PragmaShaderStage')
class TestPSSfromIncludedFile(expect.ValidObjectFile):
    """Tests that #pragma shader_stage() from included files works."""

    environment = Directory('.', [
        File('a.glsl', '#version 140\n#include "b.glsl"\n'
             'void main() { gl_Position = vec4(1.); }\n'),
        File('b.glsl', '#pragma shader_stage(vertex)')])
    glslc_args = ['-c', 'a.glsl']


@inside_glslc_testsuite('PragmaShaderStage')
class TestConflictingPSSfromIncludingAndIncludedFile(expect.ErrorMessage):
    """Tests that conflicting #pragma shader_stage() from including and
    included files results in an error with the correct location spec."""

    environment = Directory('.', [
        File('a.vert',
             '#version 140\n'
             '#pragma shader_stage(fragment)\n'
             'void main() { gl_Position = vec4(1.); }\n'
             '#include "b.glsl"\n'),
        File('b.glsl', '#pragma shader_stage(vertex)')])
    glslc_args = ['-c', 'a.vert']

    expected_error = [
        "b.glsl:1: error: '#pragma': conflicting stages for 'shader_stage' "
        "#pragma: 'vertex' (was 'fragment' at a.vert:2)\n"]


@inside_glslc_testsuite('PragmaShaderStage')
class TestPSSWithFileNameBasedPoundLine(expect.ErrorMessage):
    """Tests that #pragma shader_stage() works with filename-based #line."""

    shader = FileShader(
        """#version 310 es
        #pragma shader_stage(unknown)
        #line 42 "abc"
        #pragma shader_stage(google)
        #line 100 "def"
        #pragma shader_stage(elgoog)
        void main() {
            gl_Position = vec4(1.);
        }
        """, '.glsl')
    glslc_args = ['-c', shader]

    expected_error = [
        shader, ":2: error: '#pragma': invalid stage for 'shader_stage' "
        "#pragma: 'unknown'\n",
        "abc:42: error: '#pragma': conflicting stages for 'shader_stage' "
        "#pragma: 'google' (was 'unknown' at ", shader, ':2)\n',
        "def:100: error: '#pragma': conflicting stages for 'shader_stage' "
        "#pragma: 'elgoog' (was 'unknown' at ", shader, ':2)\n']



# Copyright 2015 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader


def assembly_comments():
    return """
    ; SPIR-V
    ; Version: 1.0
    ; Generator: Google Shaderc over Glslang; 11
    ; Bound: 6
    ; Schema: 0"""


def empty_main_assembly():
    return assembly_comments() + """
         OpCapability Shader
    %1 = OpExtInstImport "GLSL.std.450"
         OpMemoryModel Logical GLSL450
         OpEntryPoint Vertex %4 "main"
         OpSource ESSL 310
         OpName %4 "main"
    %2 = OpTypeVoid
    %3 = OpTypeFunction %2
    %4 = OpFunction %2 None %3
    %5 = OpLabel
         OpReturn
         OpFunctionEnd"""


def empty_main():
    return '#version 310 es\nvoid main() {}'


@inside_glslc_testsuite('SpirvAssembly')
class TestAssemblyFileAsOnlyParameter(expect.ValidNamedObjectFile):
    """Tests that glslc accepts a SPIR-V assembly file as the only parameter."""

    shader = FileShader(empty_main_assembly(), '.spvasm')
    glslc_args = [shader]
    expected_object_filenames = ('a.spv',)


@inside_glslc_testsuite('SpirvAssembly')
class TestDashCAssemblyFile(expect.ValidObjectFile):
    """Tests that -c works with SPIR-V assembly file."""

    shader = FileShader(empty_main_assembly(), '.spvasm')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('SpirvAssembly')
class TestAssemblyFileWithOnlyComments(expect.ValidObjectFile):
    """Tests that glslc accepts an assembly file with only comments inside."""

    shader = FileShader(assembly_comments(), '.spvasm')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('SpirvAssembly')
class TestEmptyAssemblyFile(expect.ValidObjectFile):
    """Tests that glslc accepts an empty assembly file."""

    shader = FileShader('', '.spvasm')
    glslc_args = ['-c', shader]


@inside_glslc_testsuite('SpirvAssembly')
class TestDashEAssemblyFile(expect.SuccessfulReturn, expect.NoGeneratedFiles):
    """Tests that -E works with SPIR-V assembly file."""

    shader = FileShader(empty_main_assembly(), '.spvasm')
    glslc_args = ['-E', shader]


@inside_glslc_testsuite('SpirvAssembly')
class TestDashSAssemblyFile(expect.SuccessfulReturn, expect.NoGeneratedFiles):
    """Tests that -S works with SPIR-V assembly file."""

    shader = FileShader(empty_main_assembly(), '.spvasm')
    glslc_args = ['-S', shader]


@inside_glslc_testsuite('SpirvAssembly')
class TestMultipleAssemblyFiles(expect.ValidObjectFile):
    """Tests that glslc accepts multiple SPIR-V assembly files."""

    shader1 = FileShader(empty_main_assembly(), '.spvasm')
    shader2 = FileShader(empty_main_assembly(), '.spvasm')
    shader3 = FileShader(empty_main_assembly(), '.spvasm')
    glslc_args = ['-c', shader1, shader2, shader3]


@inside_glslc_testsuite('SpirvAssembly')
class TestHybridInputFiles(expect.ValidObjectFile):
    """Tests that glslc accepts a mix of SPIR-V assembly files and
    GLSL source files."""

    shader1 = FileShader(empty_main_assembly(), '.spvasm')
    shader2 = FileShader(empty_main(), '.vert')
    shader3 = FileShader(empty_main(), '.frag')
    glslc_args = ['-c', shader1, shader2, shader3]


@inside_glslc_testsuite('SpirvAssembly')
class TestShaderStageWithAssemblyFile(expect.ErrorMessage):
    """Tests that assembly files don't work with -fshader-stage"""

    shader = FileShader(empty_main_assembly(), '.spvasm')
    glslc_args = ['-c', '-fshader-stage=vertex', shader]

    expected_error = [
        shader, ": error: #version: Desktop shaders for Vulkan SPIR-V require "
        "version 140 or higher\n",
        shader, ":2: error: 'extraneous semicolon' :",
        " not supported for this version or the enabled extensions\n",
        shader, ":2: error: '' :  syntax error, unexpected IDENTIFIER\n",
        '3 errors generated.\n']


@inside_glslc_testsuite('SpirvAssembly')
class TestStdWithAssemblyFile(expect.ValidObjectFile):
    """Tests that --std= doesn't affect the processing of assembly files."""

    shader = FileShader(empty_main_assembly(), '.spvasm')
    glslc_args = ['-c', '-std=310es', shader]



#!/usr/bin/env python3
# Copyright 2015 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Adds copyright notices to all the files that need them under the current
directory.

usage: add_copyright.py [--check]

With --check, prints out all the files missing the copyright notice and exits
with status 1 if any such files are found, 0 if none.
"""

import fileinput
import fnmatch
import os
import re
import sys

COPYRIGHTRE = re.compile(
    r'Copyright \d+ The Shaderc Authors. All rights reserved.')
COPYRIGHT = 'Copyright 2016 The Shaderc Authors. All rights reserved.'
LICENSED = """
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""


def find(top, filename_glob, skip_glob_list):
    """Returns files in the tree rooted at top matching filename_glob but not
    in directories matching skip_glob_list."""

    file_list = []
    for path, dirs, files in os.walk(top):
        for glob in skip_glob_list:
            for match in fnmatch.filter(dirs, glob):
                dirs.remove(match)
        for filename in fnmatch.filter(files, filename_glob):
            file_list.append(os.path.join(path, filename))
    return file_list


def filtered_descendants(glob):
    """Returns glob-matching filenames under the current directory, but skips
    some irrelevant paths."""
    return find('.', glob, ['third_party', 'external', 'build*', 'out*',
                            'CompilerIdCXX', '.venv', 'glslang'])


def skip(line):
    """Returns true if line is all whitespace or shebang."""
    stripped = line.lstrip()
    return stripped == '' or stripped.startswith('#!')


def comment(text, prefix):
    """Returns commented-out text.

    Each line of text will be prefixed by prefix and a space character.
    Any trailing whitespace will be trimmed.
    """
    accum = []
    for line in text.split('\n'):
        accum.append((prefix + ' ' + line).rstrip())
    return '\n'.join(accum)


def insert_copyright(glob, comment_prefix):
    """Finds all glob-matching files under the current directory and inserts
    the copyright message into them unless they already have it or are empty.

    The copyright message goes into the first non-whitespace, non-
    shebang line in a file.  It is prefixed on each line by
    comment_prefix and a space.
    """
    copyright = comment(COPYRIGHT, comment_prefix) + '\n'
    licensed = comment(LICENSED, comment_prefix) + '\n\n'
    for file in filtered_descendants(glob):
        has_copyright = False
        for line in fileinput.input(file, inplace=1):
            has_copyright = has_copyright or COPYRIGHTRE.search(line)
            if not has_copyright and not skip(line):
                sys.stdout.write(copyright)
                sys.stdout.write(licensed)
                has_copyright = True
            sys.stdout.write(line)
        if not has_copyright:
            open(file, 'a').write(copyright + licensed)


def alert_if_no_copyright(glob, comment_prefix):
    """Prints names of all files missing a copyright message.

    Finds all glob-matching files under the current directory and checks if they
    contain the copyright message.  Prints the names of all the files that
    don't.

    Returns the total number of file names printed.
    """
    printed_count = 0
    for file in filtered_descendants(glob):
        has_copyright = False
        with open(file) as contents:
            for line in contents:
                if COPYRIGHTRE.search(line):
                    has_copyright = True
                    break
        if not has_copyright:
            print(file, ' has no copyright message.')
            printed_count += 1
    return printed_count


def main():
    glob_comment_pairs = [('*.h', '//'), ('*.hpp', '//'), ('*.cc', '//'),
                          ('*.py', '#'), ('*.cpp', '//')]
    if '--check' in sys.argv:
        count = 0
        for pair in glob_comment_pairs:
            count += alert_if_no_copyright(*pair)
        sys.exit(count > 0)
    else:
        for pair in glob_comment_pairs:
            insert_copyright(*pair)


if __name__ == '__main__':
    main()



# Copyright 2015 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A number of placeholders and their rules for expansion when used in tests.

These placeholders, when used in glslc_args or expected_* variables of
GlslCTest, have special meanings. In glslc_args, they will be substituted by
the result of instantiate_for_glslc_args(), while in expected_*, by
instantiate_for_expectation(). A TestCase instance will be passed in as
argument to the instantiate_*() methods.
"""
import os
import tempfile
from string import Template
from builtins import bytes


class PlaceHolderException(Exception):
    """Exception class for PlaceHolder."""
    pass


class PlaceHolder(object):
    """Base class for placeholders."""

    def instantiate_for_glslc_args(self, testcase):
        """Instantiation rules for glslc_args.

        This method will be called when the current placeholder appears in
        glslc_args.

        Returns:
            A string to replace the current placeholder in glslc_args.
        """
        raise PlaceHolderException('Subclass should implement this function.')

    def instantiate_for_expectation(self, testcase):
        """Instantiation rules for expected_*.

        This method will be called when the current placeholder appears in
        expected_*.

        Returns:
            A string to replace the current placeholder in expected_*.
        """
        raise PlaceHolderException('Subclass should implement this function.')


class FileShader(PlaceHolder):
    """Stands for a shader whose source code is in a file."""

    def __init__(self, source, suffix, assembly_substr=None):
        assert isinstance(source, str)
        assert isinstance(suffix, str)
        self.source = source
        self.suffix = suffix
        self.filename = None
        # If provided, this is a substring which is expected to be in
        # the disassembly of the module generated from this input file.
        self.assembly_substr = assembly_substr

    def instantiate_for_glslc_args(self, testcase):
        """Creates a temporary file and writes the source into it.

        Returns:
            The name of the temporary file.
        """
        shader, self.filename = tempfile.mkstemp(
            dir=testcase.directory, suffix=self.suffix)
        shader_object = os.fdopen(shader, 'w')
        shader_object.write(self.source)
        shader_object.close()
        return self.filename

    def instantiate_for_expectation(self, testcase):
        assert self.filename is not None
        return self.filename


class StdinShader(PlaceHolder):
    """Stands for a shader whose source code is from stdin."""

    def __init__(self, source):
        assert isinstance(source, str)
        self.source = source
        self.filename = None

    def instantiate_for_glslc_args(self, testcase):
        """Writes the source code back to the TestCase instance."""
        testcase.stdin_shader = bytes(self.source, 'utf-8')
        self.filename = '-'
        return self.filename

    def instantiate_for_expectation(self, testcase):
        assert self.filename is not None
        return self.filename


class TempFileName(PlaceHolder):
    """Stands for a temporary file's name."""

    def __init__(self, filename):
        assert isinstance(filename, str)
        assert filename != ''
        self.filename = filename

    def instantiate_for_glslc_args(self, testcase):
        return os.path.join(testcase.directory, self.filename)

    def instantiate_for_expectation(self, testcase):
        return os.path.join(testcase.directory, self.filename)


class SpecializedString(PlaceHolder):
    """Returns a string that has been specialized based on TestCase.

    The string is specialized by expanding it as a string.Template
    with all of the specialization being done with each $param replaced
    by the associated member on TestCase.
    """

    def __init__(self, filename):
        assert isinstance(filename, str)
        assert filename != ''
        self.filename = filename

    def instantiate_for_glslc_args(self, testcase):
        return Template(self.filename).substitute(vars(testcase))

    def instantiate_for_expectation(self, testcase):
        return Template(self.filename).substitute(vars(testcase))




# Copyright 2016 The Shaderc Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import expect
from environment import File, Directory
from glslc_test_framework import inside_glslc_testsuite
from placeholder import FileShader

MINIMAL_SHADER = '#version 310 es\nvoid main() {}'
EMPTY_SHADER_IN_CWD = Directory('.', [File('shader.vert', MINIMAL_SHADER)])

ASSEMBLY_WITH_DEBUG_SOURCE = [
    """; SPIR-V
; Version: 1.0
; Generator: Google Shaderc over Glslang; 11
; Bound: 7
; Schema: 0
               OpCapability Shader
          %2 = OpExtInstImport "GLSL.std.450"
               OpMemoryModel Logical GLSL450
               OpEntryPoint Vertex %main "main"
          %1 = OpString "shader.vert"
               OpSource ESSL 310 %1 "// OpModuleProcessed entry-point main
// OpModuleProcessed client vulkan100
// OpModuleProcessed target-env vulkan1.0
// OpModuleProcessed entry-point main
#line 1
#version 310 es
void main() {}"
               OpSourceExtension "GL_GOOGLE_cpp_style_line_directive"
               OpSourceExtension "GL_GOOGLE_include_directive"
               OpName %main "main"
       %void = OpTypeVoid
          %4 = OpTypeFunction %void
               OpLine %1 2 11
       %main = OpFunction %void None %4
          %6 = OpLabel
               OpLine %1 2 0
               OpReturn
               OpFunctionEnd
"""
]

ASSEMBLY_O0 = [
    """; SPIR-V
; Version: 1.0
; Generator: Google Shaderc over Glslang; 11
; Bound: 6
; Schema: 0
               OpCapability Shader
          %1 = OpExtInstImport "GLSL.std.450"
               OpMemoryModel Logical GLSL450
               OpEntryPoint Vertex %main "main"
               OpSource ESSL 310
               OpSourceExtension "GL_GOOGLE_cpp_style_line_directive"
               OpSourceExtension "GL_GOOGLE_include_directive"
               OpName %main "main"
       %void = OpTypeVoid
          %3 = OpTypeFunction %void
       %main = OpFunction %void None %3
          %5 = OpLabel
               OpReturn
               OpFunctionEnd
"""]

ASSEMBLY_O = [
    """; SPIR-V
; Version: 1.0
; Generator: Google Shaderc over Glslang; 11
; Bound: 6
; Schema: 0
               OpCapability Shader
          %1 = OpExtInstImport "GLSL.std.450"
               OpMemoryModel Logical GLSL450
               OpEntryPoint Vertex %4 "main"
       %void = OpTypeVoid
          %3 = OpTypeFunction %void
          %4 = OpFunction %void None %3
          %5 = OpLabel
               OpReturn
               OpFunctionEnd
"""]

ASSEMBLY_Os = [
    """; SPIR-V
; Version: 1.0
; Generator: Google Shaderc over Glslang; 11
; Bound: 6
; Schema: 0
               OpCapability Shader
          %1 = OpExtInstImport "GLSL.std.450"
               OpMemoryModel Logical GLSL450
               OpEntryPoint Vertex %4 "main"
       %void = OpTypeVoid
          %3 = OpTypeFunction %void
          %4 = OpFunction %void None %3
          %5 = OpLabel
               OpReturn
               OpFunctionEnd
"""]


@inside_glslc_testsuite('OptionDashCapO')
class TestDashCapO0(expect.ValidFileContents):
    """Tests that -O0 works."""

    environment = EMPTY_SHADER_IN_CWD
    glslc_args = ['-S', '-O0', 'shader.vert']
    target_filename = 'shader.vert.spvasm'
    expected_file_contents = ASSEMBLY_O0


@inside_glslc_testsuite('OptionDashCapO')
class TestDashCapOPerformance(expect.ValidFileContents):
    """Tests -O works."""

    environment = EMPTY_SHADER_IN_CWD
    glslc_args = ['-S', '-O', 'shader.vert']
    target_filename = 'shader.vert.spvasm'
    expected_file_contents = ASSEMBLY_O


@inside_glslc_testsuite('OptionDashCapO')
class TestDashCapOs(expect.ValidFileContents):
    """Tests that -Os works."""

    environment = EMPTY_SHADER_IN_CWD
    glslc_args = ['-S', '-Os', 'shader.vert']
    target_filename = 'shader.vert.spvasm'
    expected_file_contents = ASSEMBLY_Os


@inside_glslc_testsuite('OptionDashCapO')
class TestDashCapOOverriding(expect.ValidFileContents):
    """Tests that if there are multiple -O's, only the last one takes effect."""

    environment = EMPTY_SHADER_IN_CWD
    glslc_args = ['-S', '-Os', '-O0', '-Os', '-O0', 'shader.vert']
    target_filename = 'shader.vert.spvasm'
    expected_file_contents = ASSEMBLY_O0


@inside_glslc_testsuite('OptionDashCapO')
class TestDashCapOWithDashG(expect.ValidFileContents):
    """Tests that -g restrains -O from turning on strip debug info."""

    environment = EMPTY_SHADER_IN_CWD
    glslc_args = ['-S', '-Os', '-g', 'shader.vert']
    target_filename = 'shader.vert.spvasm'
    expected_file_contents = ASSEMBLY_WITH_DEBUG_SOURCE


@inside_glslc_testsuite('OptionDashCapO')
class TestDashGWithDashCapO(expect.ValidFileContents):
    """Tests that -g restrains -O from turning on strip debug info."""

    environment = EMPTY_SHADER_IN_CWD
    glslc_args = ['-S', '-g', '-Os', 'shader.vert']
    target_filename = 'shader.vert.spvasm'
    expected_file_contents = ASSEMBLY_WITH_DEBUG_SOURCE


@inside_glslc_testsuite('OptionDashCapO')
class TestWrongOptLevel(expect.NoGeneratedFiles, expect.ErrorMessage):
    """Tests erroring out with wrong optimization level."""

    shader = FileShader(MINIMAL_SHADER, '.vert')
    glslc_args = ['-c', '-O2', shader]
    expected_error = "glslc: error: invalid value '2' in '-O2'\n"
